"""
This module contains a JinjaRenderer class that can be used to add a
renderer property to the context that renders Jinja2 templates.

This extension requires the Jinja2 module to be installed::

    $ pip install jinja2

Example usage::

    from rhino import Mapper, get
    from rhino.ext.jinja2 import JinjaRenderer

    app = rhino.Mapper()
    app.add_ctx_property('render_template', JinjaRenderer('./templates'))

    @get
    def index(request, ctx):
        return ctx.render_template('index.html', greeting="hello, world!")

    app.add('/', index)

"""
from __future__ import absolute_import

from functools import partial

import jinja2
from rhino.response import Entity


class JinjaRenderer(object):
    encoding = 'utf-8'
    content_type = 'text/html; charset=utf-8'

    def __init__(self, directory=None, autoescape=True, **env_args):
        if directory is not None:
            env_args['loader'] = jinja2.FileSystemLoader(directory)
        self.env = jinja2.Environment(autoescape=autoescape,  **env_args)

    def render_template(self, ctx, template_name, **values):
        template = self.env.get_template(template_name)
        body = template.render(ctx=ctx, url_for=ctx.request.url_for, **values)
        return Entity(
                body=body.encode(self.encoding),
                content_type=self.content_type)

    def __call__(self, ctx):
        return partial(self.render_template, ctx)



import os
import shutil
import tempfile

import pytest
from rhino.test import TestClient

from examples.wiki.wiki import app

@pytest.fixture
def client(request):
    tmpdir = tempfile.mkdtemp(prefix='rhino_test')
    cwd = os.getcwd()
    os.chdir(tmpdir)
    os.mkdir('contents')

    def finalize():
        os.chdir(cwd)
        shutil.rmtree(tmpdir)

    request.addfinalizer(finalize)
    return TestClient(app.wsgi)


def test_frontpage(client):
    status, headers, body = client.get('/')
    assert headers['Location'] == 'http://localhost/FrontPage'

    status, headers, body = client.get('/FrontPage')
    assert headers['Content-Type'] == 'text/html; charset=utf-8'
    assert 'FrontPage' in body


def test_edit(client):
    status, headers, body = client.get('/FrontPage/edit')
    assert 'Editing FrontPage' in body

    status, headers, body = client.post('/FrontPage/edit', {'content': 'xyz', 'submit': 1})
    assert status == "302 Found"
    assert headers['Location'] == 'http://localhost/FrontPage'

    status, headers, body = client.get('/FrontPage')
    assert 'xyz' in body


def test_history(client):
    client.post('/FrontPage/edit', {'content': 'test1', 'submit': 1})
    client.post('/FrontPage/edit', {'content': 'test2', 'submit': 1})

    status, headers, body = client.get('/FrontPage')
    assert 'test2' in body

    status, headers, body = client.get('/FrontPage/history')
    assert 'History For FrontPage' in body

    status, headers, body = client.post('/FrontPage/history', {'version': 1, 'submit': 1})
    assert status == "302 Found"
    assert headers['Location'] == 'http://localhost/FrontPage'

    status, headers, body = client.get('/FrontPage')
    assert 'test1' in body



import os

from rhino import Mapper, StaticFile, StaticDirectory, get
from rhino.errors import NotFound
from rhino.ext.jinja2 import JinjaRenderer
from rhino.ext.sqlalchemy import SessionProperty

from model import Movie, Actor

APP_ROOT = os.path.dirname(__file__)


@get
def index(request, ctx):
    movies = ctx.db.query(Movie).order_by(Movie.title.asc())
    return ctx.render_template('index.html', movies=movies)


@get
def show_movie(request, ctx, id):
    movie = ctx.db.query(Movie).get(int(id))
    if not movie:
        raise NotFound
    return ctx.render_template('movie.html', movie=movie)


@get
def show_actor(request, ctx, id):
    actor = ctx.db.query(Actor).get(int(id))
    if not actor:
        raise NotFound
    return ctx.render_template('actor.html', actor=actor)


def get_app(db_url):
    app = Mapper()

    template_dir = os.path.join(APP_ROOT, 'templates')
    static_dir = os.path.join(APP_ROOT, 'static')
    favicon = os.path.join(static_dir, 'favicon.ico')

    app.add_ctx_property('db', SessionProperty(db_url))
    app.add_ctx_property('render_template', JinjaRenderer(template_dir))

    app.add('/', index)
    app.add('/movies/{id}', show_movie, name='movie')
    app.add('/actors/{id}', show_actor, name='actor')
    app.add('/favicon.ico', StaticFile(favicon))
    app.add('/static/{path:any}', StaticDirectory(static_dir), name='static')
    return app

if __name__ == '__main__':
    import os
    from model import init_db

    if not os.path.exists('movies.db'):
        init_db('sqlite:///movies.db')

    app = get_app('sqlite:///movies.db')
    app.start_server()



"""
This module contains classes that can be used to add session support for Rhino
applications.

This extension requires the Beaker module to be installed::

    $ pip install beaker

The Session class provides a simple and easy to use signed cookie-based session
with support for displaying "flashed" messages.

The BeakerSession class provides access to the different backends provided by
beaker.session.
"""
from __future__ import absolute_import

from collections import namedtuple
from functools import partial

import beaker.session
from beaker.util import coerce_session_params

message = namedtuple('message', 'type text')


class SessionObject(beaker.session.SessionObject):
    """A session object with support for "flashed" messages."""

    _msg_key = '_msg'

    def add_message(self, text, type=None):
        """Add a message with an optional type."""
        key = self._msg_key
        self.setdefault(key, [])
        self[key].append(message(type, text))
        self.save()

    def pop_messages(self, type=None):
        """Retrieve stored messages and remove them from the session.

        Return all messages with a specific type, or all messages when `type`
        is None. Messages are returned in the order they were added. All
        messages returned in this way are removed from the session and will not
        be returned in subsequent calls.

        Returns a list of namedtuples with the fields (type, text).
        """
        key = self._msg_key
        messages = []
        if type is None:
            messages = self.pop(key, [])
        else:
            keep_messages = []
            for msg in self.get(key, []):
                if msg.type == type:
                    messages.append(msg)
                else:
                    keep_messages.append(msg)
            if not keep_messages and key in self:
                del self[key]
            else:
                self[key] = keep_messages
        if messages:
            self.save()

        return messages


class BeakerSession(object):
    """Adds a session property to the context."""
    session_class = SessionObject

    def __init__(self, **session_args):
        # Default parameters from beaker.middleware.SessionMiddleware
        self.options = dict(invalidate_corrupt=True, type=None,
                            data_dir=None, key='beaker.session.id',
                            timeout=None, secret=None, log_file=None)
        self.options.update(session_args)
        coerce_session_params(self.options)

    def finalize(self, session, request, response):
        if session.accessed():
            session.persist()
            if session.__dict__['_headers']['set_cookie']:
                cookie = session.__dict__['_headers']['cookie_out']
                if cookie:
                    response.headers.add_header('Set-Cookie', cookie)

    # TODO Since context properties are lazily initialized by default, the
    # session will not be loaded at all unless it is accessed at least once
    # during the request.
    def __call__(self, ctx):
        session = self.session_class(ctx.request.environ, **self.options)
        ctx.add_callback('finalize', partial(self.finalize, session))
        return session


class Session(BeakerSession):
    """Adds a session based on signed cookies to the context."""
    session_class = SessionObject

    # Avoid passing **kwargs to Beaker because it silently ignores unknown
    # arguments -- bad when you have a typo (e.g. htponly vs httponly).
    def __init__(
            self, secret, timeout=None, cookie_name='session_id',
            cookie_expires=True, cookie_domain=None, cookie_path='/',
            secure=False, httponly=False, auto=True):
        super(Session, self).__init__(
            type='cookie',
            validate_key=secret,
            key=cookie_name,
            timeout=timeout,
            cookie_expires=cookie_expires,
            cookie_domain=cookie_domain,
            cookie_path=cookie_path,
            secure=secure,
            httponly=httponly,
            auto=auto,
        )



import os
import shutil
import tempfile

import pytest
from pytest import raises as assert_raises
from rhino.mapper import Mapper
from rhino.static import StaticFile, StaticDirectory
from rhino.test import TestClient


@pytest.fixture(scope='module')
def tmpdir(request):
    tmpdir = tempfile.mkdtemp(prefix='rhino_test')
    request.addfinalizer(lambda: shutil.rmtree(tmpdir))
    return tmpdir


@pytest.fixture(scope='module')
def app(tmpdir):
    robots_txt = os.path.join(tmpdir, 'robots.txt')
    with open(robots_txt, 'w') as f:
        f.write('robots.txt')

    index_html = os.path.join(tmpdir, 'index.html')
    with open(index_html, 'w') as f:
        f.write('index.html')

    binary_file = os.path.join(tmpdir, 'file')
    with open(binary_file, 'w') as f:
        f.write('\0')

    app = Mapper()
    app.add('/robots.txt', StaticFile(robots_txt, expires=60), 'robots')
    app.add('/somefile.bin', StaticFile(binary_file), 'binary')
    app.add('/static/{path:any}', StaticDirectory(tmpdir, expires=120), 'static')
    return app


@pytest.fixture
def client(app):
    client = TestClient(app.wsgi)
    return client


def test_add_invalid_file():
    assert_raises(ValueError, StaticFile, 'file.does.not.exist')


def test_plain(client):
    res = client.get('/robots.txt')
    assert res.status == "200 OK"
    assert res.body == "robots.txt"
    assert res.headers['Content-Type'] == "text/plain"
    assert res.headers['Content-Length'] == "10"
    assert 'Expires' in res.headers

    res = client.post('/robots.txt', {})
    assert res.status == "405 Method Not Allowed"
    assert res.headers['Allow'] == 'GET, HEAD'


def test_binary(client):
    res = client.get('/somefile.bin')
    assert res.status == "200 OK"
    assert res.body == '\0'
    assert res.headers['Content-Type'] == "application/octet-stream"
    assert res.headers['Content-Length'] == "1"

    res = client.post('/robots.txt', {})
    assert res.status == "405 Method Not Allowed"
    assert res.headers['Allow'] == 'GET, HEAD'


def test_static_dir(client):
    res = client.get('/static/index.html')
    assert res.status == "200 OK"
    assert res.body == "index.html"
    assert res.headers['Content-Type'] == "text/html"
    assert res.headers['Content-Length'] == "10"
    assert 'Expires' in res.headers

    res = client.get('/static/')
    assert res.status == "404 Not Found"
    assert 'Expires' not in res.headers

    res = client.get('/static/does_not_exist')
    assert res.status == "404 Not Found"


def test_no_pathname_leak(client, tmpdir):
    res = client.get('/static/index.html')
    assert res.status == "200 OK"

    dirname = os.path.basename(tmpdir)
    res = client.get('/static/../%s/index.html' % dirname)
    assert res.status == "404 Not Found"
    res = client.get('/static/foo/../../%s/index.html' % dirname)
    assert res.status == "404 Not Found"



from rhino.test import TestClient

from examples.helloworld import app

client = TestClient(app.wsgi)


def test_get():
    res = client.get('/')
    assert res.code == 200
    assert res.headers.items() == [
        ('Content-Type', 'text/plain; charset=utf-8'),
        ('Content-Length', '13'),
    ]
    assert res.body == 'Hello, world!'


def test_head():
    res = client.head('/')
    assert res.code == 200
    assert res.headers.items() == [
        ('Content-Type', 'text/plain; charset=utf-8'),
        ('Content-Length', '13'),
    ]
    assert res.body == ''


def test_options():
    res = client.options('/')
    assert res.code == 200
    assert res.headers['Allow'] == 'GET, HEAD, OPTIONS'


def test_delete():
    res = client.delete('/')
    assert res.code == 405
    assert res.headers['Allow'] == 'GET, HEAD, OPTIONS'



import sqlalchemy as sa
from sqlalchemy.ext.declarative import declarative_base
from sqlalchemy.orm import Session, relationship, backref

Base = declarative_base()


class Movie(Base):
    __tablename__ = 'movies'

    id = sa.Column(sa.Integer, primary_key=True)
    title = sa.Column(sa.String)
    rating = sa.Column(sa.Integer, default=0)

    roles = relationship('Role', backref='movie', order_by='Role.name')

    def __init__(self, title):
        self.title = title


class Actor(Base):
    __tablename__ = 'actors'

    id = sa.Column(sa.Integer, primary_key=True)
    name = sa.Column(sa.String)

    roles = relationship('Role', backref='actor', order_by='Role.name')

    def __init__(self, name):
        self.name = name


class Role(Base):
    __tablename__ = 'roles'

    id = sa.Column(sa.Integer, primary_key=True)
    name = sa.Column(sa.String)
    actor_id = sa.Column(sa.Integer, sa.ForeignKey('actors.id'))
    movie_id = sa.Column(sa.Integer, sa.ForeignKey('movies.id'))

    def __init__(self, name, actor):
        self.name = name
        self.actor = actor


def init_db(url):
    engine = sa.create_engine(url)
    Base.metadata.create_all(engine)

    session = Session(bind=engine)

    starwars = Movie("Star Wars")
    starwars.rating = 5
    starwars.roles = [
        Role("Luke Skywalker", Actor("Mark Hamill")),
        Role("Leia Organa", Actor("Carrie Fisher")),
        Role("Obi-Wan Kenobi", Actor("Sir Alec Guiness")),
        Role("Han Solo", Actor("Harrison Ford")),
    ]
    session.add(starwars)
    session.commit()
    session.close()



from __future__ import absolute_import

import urllib
import urlparse
import time
from Cookie import SimpleCookie
from datetime import datetime, timedelta
from wsgiref.headers import Headers as ResponseHeaders
from wsgiref.util import application_uri

from .http import httpdate_to_timestamp, datetime_to_httpdate, \
        timedelta_to_httpdate, total_seconds, match_etag, status_codes

# Include: etag, content-location, expires, cache-control, vary
# TODO The revised HTTP RFCs (rfcs 7230-7235) have removed the distinction
# between entity-headers and other heathers. We might need to revisit this.
entity_headers = set('''
       Allow Content-Encoding Content-Language Content-Length Content-Location
       Content-MD5 Content-Range Content-Type Expires Last-Modified
    '''.lower().split())

_include_in_304 = set('''
        Date ETag Content-Location Expires Cache-Control Vary
    '''.lower().split())

_filter_from_304 = entity_headers - _include_in_304


def filter_304_headers(headers):
    """Filter a list of headers to include in a "304 Not Modified" response."""
    return [(k, v) for k, v in headers if k.lower() not in _filter_from_304]


class Entity(object):
    """Represents a response body with entity headers."""

    def __init__(self, body, **kw):
        self.body = body
        self.headers = response(200, **kw).headers


class ResponseBody(object):
    """A WSGI response iterator.

    Holds the response body and a list of callbacks to be called when the
    response is close()d by the WSGI server.
    """

    def __init__(self, body, callbacks=None):
        if callbacks is None:
            callbacks = []
        if not hasattr(body, '__iter__'):
            body = iter([body])
        self.callbacks = callbacks
        self.body = body

    def __iter__(self):
        return self

    def next(self):
        return next(self.body)

    def close(self):
        for fn in self.callbacks:
            fn()


class Response(object):
    """Represents an HTTP response.

    Class attributes:

    default_encoding
        When finalizing the response and the response body is a unicode string,
        it is encoded using default_encoding (default: 'utf-8')

    default_content_type
        When finalizing the response and the response body is not empty this is
        used as the default value for the Content-Type header, if none is
        provided (default: 'text/plain; charset=utf-8')
    """
    default_encoding = 'utf-8'
    default_content_type = 'text/plain; charset=utf-8'

    def __init__(self, code, headers=None, body=''):
        """Create a new HTTP response.

        Parameters:

        code
            an integer status code.

        headers
            a list of response headers as (name, value) tuples.

        body
            a value for the response body.
            The body is only validated when the request is finalized
            (in __call__).

            Valid values for the response body are:

            The empty string ('')
                This indicates an empty response body.

            A str or unicode object
               Sent as-is, after encoding unicode using default_encoding

            An iterator that yields str or unicode objects
               The response is streamed to the client using chunked
               transfer-encoding (when implemented by the WSGI server).

            A callable that returns a single str or unicode object
               This is only useful in combination with an 'Etag' or
               'Last-Modified' header, to delay construction of the
               response body until after conditional request handling
               has taken place, and no "304 Not Modified" response has
               been sent.

            An Entity instance
                The response body (which must be of one of the types listed
                above) will be taken from the entity instance, and any entity
                headers will be added to the response headers, with existing
                headers of the same name taking precedence.

        """
        headers = ResponseHeaders(headers or [])

        if isinstance(body, Entity):
            entity = body
            body = entity.body
            # Response headers override entity headers
            for k, v in entity.headers.items():
                headers.setdefault(k, v)

        self._code = code
        self._headers = headers
        self._body = body
        self._callbacks = []
        self.request = None

    @property
    def code(self):
        """The HTTP status code as an integer."""
        return self._code

    @property
    def headers(self):
        """A ResponseHeaders instance representing the response headers."""
        return self._headers

    @property
    def body(self):
        """The response body."""
        return self._body

    @property
    def status(self):
        """The statusline as a string (status code + reason)"""
        reason = status_codes.get(self.code, "Unknown")
        return "%s %s" % (self._code, reason)

    def add_callback(self, fn):
        """Add a callback to be executed when the response is closed."""
        self._callbacks.append(fn)

    def set_cookie(self, key, value='', max_age=None, path='/', domain=None,
                   secure=False, httponly=False, expires=None):
        """Set a response cookie.

        Parameters:

        key
            The cookie name.
        value
            The cookie value.
        max_age
            The maximum age of the cookie in seconds, or as a
            datetime.timedelta instance.
        path
            Restrict the cookie to this path (default: '/').
        domain
            Restrict the cookie to his domain.
        secure
            When True, instruct the client to only sent the cookie over HTTPS.
        httponly
            When True, instruct the client to disallow javascript access to
            the cookie.
        expires:
            Another way of specifying the maximum age of the cookie. Accepts
            the same values as max_age (number of seconds, datetime.timedelta).
            Additionaly accepts a datetime.datetime instance.
            Note: a value of type int or float is interpreted as a number of
            seconds in the future, *not* as Unix timestamp.
        """
        key, value = key.encode('utf-8'), value.encode('utf-8')
        cookie = SimpleCookie({key: value})
        m = cookie[key]
        if max_age is not None:
            if isinstance(max_age, timedelta):
                m['max-age'] = int(total_seconds(max_age))
            else:
                m['max-age'] = int(max_age)
        if path is not None: m['path'] = path.encode('utf-8')
        if domain is not None: m['domain'] = domain.encode('utf-8')
        if secure: m['secure'] = True
        if httponly: m['httponly'] = True
        if expires is not None:
            # 'expires' expects an offset in seconds, like max-age
            if isinstance(expires, datetime):
                expires = total_seconds(expires - datetime.utcnow())
            elif isinstance(expires, timedelta):
                expires = total_seconds(expires)
            m['expires'] = int(expires)

        self.headers.add_header('Set-Cookie', m.OutputString())

    def delete_cookie(self, key, path='/', domain=None):
        """Delete a cookie (by setting it to a blank value).

        The path and domain values must match that of the original cookie.
        """
        self.set_cookie(key, value='', max_age=0, path=path, domain=domain,
                        expires=datetime.utcfromtimestamp(0))

    def conditional_to(self, request):
        """Return a response that is conditional to a given request.

        Returns the Response instance unchanged, or a new Response instance
        with a "304 Not Modified" status code.
        """
        if not self.code == 200:
            return self

        request_headers = request.headers
        response_headers = self.headers

        if_none_match = request_headers.get('If-None-Match')
        if_modified_since = request_headers.get('If-Modified-Since')

        etag_ok, date_ok = False, False

        if if_none_match:
            etag = response_headers.get('Etag')
            if etag and match_etag(etag, if_none_match, weak=True):
                etag_ok = True

        if if_modified_since:
            last_modified = response_headers.get('Last-Modified')
            if last_modified:
                try:
                    modified_ts = httpdate_to_timestamp(last_modified)
                    last_ok_ts = httpdate_to_timestamp(if_modified_since)
                    if modified_ts <= last_ok_ts:
                        date_ok = True
                except:
                    pass

        if if_none_match and not etag_ok:
            return self
        elif if_modified_since and not date_ok:
            return self
        elif etag_ok or date_ok:
            headers = filter_304_headers(self.headers.items())
            if 'Date' not in self.headers:
                headers.append(('Date', datetime_to_httpdate(time.time())))
            return Response(code=304, headers=headers, body='')
        return self

    def __call__(self, environ, start_response):
        """WSGI interface

        Finalizes the request, calls `start_response` and returns a
        response iterator.
        """
        code = self._code
        headers = self._headers
        body = self._body

        # Resolve lazy response body
        if callable(body):
            body = body()

        # Validate response body
        if isinstance(body, unicode):
            body = body.encode(self.default_encoding)
        elif hasattr(body, '__iter__'):
            body = (s.encode(self.default_encoding)
                    if isinstance(s, unicode) else s
                    for s in body)
        elif not isinstance(body, str):
            raise TypeError("response body must be of type unicode, str,"
                            " or an iterator, not '%s'" % type(body))

        # Make sure we have Content-Type and Content-Length headers if needed.
        if code != 304:
            if 'Content-Type' not in headers:
                headers['Content-Type'] = self.default_content_type
            if type(body) is str and 'Content-Length' not in headers:
                headers['Content-Length'] = str(len(body))

        # Special case for Location header: accept unicode, make absolute.
        location = headers.get('Location')
        if location is not None:
            if isinstance(location, unicode):
                location = location.encode('utf-8')
            headers['Location'] = urlparse.urljoin(
                application_uri(environ),
                urllib.quote(location, safe=';/?:@&=+$,#')
            )

        # Send response
        header_list = [(k.encode('ascii'), v.encode('latin-1'))
                       for k, v in headers.items()]

        request_callbacks = self.request._callbacks if self.request else []
        callbacks = request_callbacks + self._callbacks

        start_response(self.status, header_list)

        if code in (204, 304) or environ.get('REQUEST_METHOD') == 'HEAD':
            return ResponseBody('', callbacks)
        return ResponseBody(body, callbacks)


def response(code, body='', etag=None, last_modified=None, expires=None, **kw):
    """Helper to build an HTTP response.

    Parameters:

    code
        An integer status code.
    body
        The response body. See Response.__init__() for details.
    etag
        A value for the Etag header. Double quotes will be added unless the
        string starts and ends with a double qote.
    last_modified
        A value for the Last-Modified header as a datetime.datetime instance
        or Unix timestamp.
    expires
        A value for the Expires header as number of seconds, datetime.timedelta
        or datetime.datetime instance.
        Note: a value of type int or float is interpreted as a number of
        seconds in the future, *not* as Unix timestamp.
    `**kw`
        All other keyword arguments are interpreted as response headers.
        The names will be converted to header names by replacing
        underscores with hyphens and converting to title case
        (e.g. 'x_powered_by' => 'X-Powered-By').

    """
    if etag is not None:
        if not (etag[0] == '"' and etag[-1] == '"'):
            etag = '"%s"' % etag
        kw['etag'] = etag

    if last_modified is not None:
        kw['last_modified'] = datetime_to_httpdate(last_modified)

    if expires is not None:
        if isinstance(expires, datetime):
            kw['expires'] = datetime_to_httpdate(expires)
        else:
            kw['expires'] = timedelta_to_httpdate(expires)

    headers = [(k.replace('_', '-').title(), v) for k, v in sorted(kw.items())]
    return Response(code, headers, body)


def ok(body='', code=200, **kw):
    """Shortcut for response(200, ...).

    The status code must be in the 2xx range."""
    if not 200 <= code < 300:
        raise ValueError("Not a 2xx status code: '%s'" % code)
    return response(code=code, body=body, **kw)


def created(body='', **kw):
    """Shortcut for response(201, ...)."""
    return response(code=201, body=body, **kw)


def no_content(**kw):
    """Shortcut for response(204, body='', ...)."""
    return response(code=204, body='', **kw)


def redirect(location, code=302, **kw):
    """Shortcut for response(302, location=location, ...)

    The status code must be in the 3xx range."""
    if not 300 <= code < 400:
        raise ValueError("Not a 3xx status code: '%s'" % code)
    return response(code=code, location=location, **kw)



from rhino.mapper import Context
from rhino.ext.sqlalchemy import SessionProperty
from sqlalchemy.orm import Session


def test_default_engine():
    session_property = SessionProperty('sqlite:///:memory:')
    assert 'bind' in session_property.session_args
    assert session_property.session_args['bind'].name == 'sqlite'


def test_property():
    ctx = Context()
    ctx.add_property('session', SessionProperty('sqlite:///:memory:'))
    session = ctx.session
    assert isinstance(session, Session)
    assert ctx._Context__callbacks['teardown'][0] == session.close


def test_delay_close():
    ctx = Context()
    ctx.add_property('session',
            SessionProperty('sqlite:///:memory:', delay_close=True))
    session = ctx.session
    assert ctx._Context__callbacks['close'][0] == session.close



from collections import defaultdict

from pytest import raises as assert_raises
from rhino.errors import NotFound
from rhino.mapper import Context
from rhino.request import Request
from rhino.resource import Resource, get, put, delete
from rhino.response import Response, ok
from rhino.test import make_handler_metadata


def test_call():
    class TestResource(object):
        pass

    resource = TestResource()
    resource.fn1 = lambda r: Response(200, headers=[('Vary', 'User-Agent')], body='test')
    resource.fn1._rhino_meta = make_handler_metadata(verb='GET', provides='text/plain')
    resource.fn2 = lambda r: Response(200, body='test')
    resource.fn2._rhino_meta = make_handler_metadata(verb='GET', provides='text/html')
    resource.fn3 = lambda r: Response(200, body='test')
    resource.fn3._rhino_meta = make_handler_metadata(verb='POST')

    ctx = Context()
    wrapped = Resource(resource)

    res = wrapped(Request({'REQUEST_METHOD': 'GET'}), ctx)
    assert res.headers['Content-Type'] == 'text/plain'
    assert res.headers['Vary'] == 'Accept, User-Agent'

    res = wrapped(Request(
        {'REQUEST_METHOD': 'GET', 'HTTP_ACCEPT': 'text/html'}), ctx)
    assert res.headers['Content-Type'] == 'text/html'
    assert res.headers['Vary'] == 'Accept'

    res = wrapped(Request(
        {'REQUEST_METHOD': 'POST', 'HTTP_ACCEPT': 'text/html'}), ctx)
    assert 'Content-Type' not in res.headers
    assert 'Vary' not in res.headers


def test_resource_emtpy():
    r = Resource(None)
    req = Request({'REQUEST_METHOD': 'GET'})
    ctx = Context()
    assert_raises(NotFound, r, req, ctx)


def test_resource_fn():
    @get
    def handler1(request):
        return ok()

    @get
    def handler2(request, ctx):
        return ok()

    r1 = Resource(handler1)
    r2 = Resource(handler2)
    req = Request({'REQUEST_METHOD': 'GET'})
    ctx = Context()
    assert r1(req, ctx)
    assert r2(req, ctx)


def test_resource_obj():
    class MyResource(object):
        @get
        def handler1(self, request):
            return ok()

        @put
        def handler2(self, request, ctx):
            return ok()

    r = Resource(MyResource())
    ctx = Context()
    assert r(Request({'REQUEST_METHOD': 'GET'}), ctx)
    assert r(Request({'REQUEST_METHOD': 'PUT'}), ctx)


def test_resource_class():
    @Resource
    class MyResource(object):
        @get
        def handler1(self, request):
            return ok()

        @put
        def handler2(self, request, ctx):
            return ok()

    ctx = Context()
    assert MyResource(Request({'REQUEST_METHOD': 'GET'}), ctx)
    assert MyResource(Request({'REQUEST_METHOD': 'PUT'}), ctx)


def test_resource_from_url():
    class ResourceWithFromUrl(object):
        def __init__(self):
            self.called = False

        def from_url(self, request):
            self.called = True
            return {}

    class ResourceWithFromUrlCtx(ResourceWithFromUrl):
        def from_url(self, request, ctx):
            self.called = True
            return {}

    class ResourceWithFromUrlError(ResourceWithFromUrl):
        def from_url(self, request):
            self.called = True
            return 'test'

    resource1 = ResourceWithFromUrl()
    resource1.handler = get(lambda req: ok())

    resource2 = ResourceWithFromUrlCtx()
    resource2.handler = get(lambda req: ok())

    resource3 = ResourceWithFromUrlError()
    resource3.handler = get(lambda req: ok())

    r1 = Resource(resource1)
    r2 = Resource(resource2)
    r3 = Resource(resource3)
    req = Request({'REQUEST_METHOD': 'GET'})
    ctx = Context()
    assert r1(req, ctx)
    assert resource1.called
    assert r2(req, ctx)
    assert resource2.called
    assert_raises(TypeError, r3, req, ctx)
    assert resource3.called



from rhino import Mapper, get
from rhino.ext.session import Session, SessionObject, message
from rhino.test import TestClient

def test_messages():
    session = SessionObject(environ={})
    session.add_message('test')
    assert session.pop_messages() == [message(None, 'test')]
    assert session.pop_messages() == []


def test_pop_all_messages():
    session = SessionObject(environ={})
    session.add_message('test')
    session.add_message('test', type='foo')
    session.add_message('test', type='bar')
    assert session.pop_messages() == [
        message(None, 'test'),
        message('foo', 'test'),
        message('bar', 'test'),
    ]
    assert session.pop_messages() == []


def test_messages_by_type():
    session = SessionObject(environ={})
    session.add_message('test')
    session.add_message('test', type='foo')
    session.add_message('test', type='bar')
    assert session.pop_messages('foo') == [message('foo', 'test')]
    assert session.pop_messages('bar') == [message('bar', 'test')]
    assert session.pop_messages() == [message(None, 'test')]
    assert session.pop_messages('foo') == []
    assert session.pop_messages('bar') == []


def test_session_property():
    @get
    def handler(request, ctx):
        ctx.session['foo'] = 'bar'
        return 'test'

    app = Mapper()
    app.add_ctx_property('session', Session(secret='test'))
    app.add('/', handler)
    client = TestClient(app.wsgi)

    res = client.get('/')
    assert 'session_id' in res.headers['Set-Cookie']



from rhino.test import TestClient

from examples.conneg import app

client = TestClient(app.wsgi)


def test_get_html():
    res = client.get('/', {'HTTP_ACCEPT': 'text/html'})
    assert res.code == 200
    assert res.headers['Content-Type'] == 'text/html'
    assert res.body == '<html><h1>Hello, world!</h1></html>'


def test_get_json():
    res = client.get('/', {'HTTP_ACCEPT': 'application/json'})
    assert res.code == 200
    assert res.headers['Content-Type'] == 'application/json'
    assert res.body == '{"greeting": "Hello, world!"}'

def test_put_json():
    res = client.put('/', '{"greeting": "Howdy!"}', content_type='application/json')
    assert res.code == 200

    res = client.get('/', {'HTTP_ACCEPT': 'application/json'})
    assert res.body == '{"greeting": "Howdy!"}'



from mock import Mock, call
from pytest import raises as assert_raises
from rhino.mapper import Context, Mapper, InvalidArgumentError
from rhino.request import Request
from rhino.response import ok


def test_add_property():
    foo = Mock(return_value=1)
    ctx = Context()
    ctx.add_property('foo', foo)
    assert ctx.foo == 1
    assert ctx.foo == 1
    foo.assert_called_once_with(ctx)


def test_invalid_property():
    ctx = Context()
    assert_raises(AttributeError, lambda: ctx.foo)


def test_add_property_duplicate():
    ctx = Context()
    ctx.add_property('foo', 1)
    assert_raises(KeyError, ctx.add_property, 'foo', 2)


def test_add_property_static():
    ctx = Context()
    ctx.add_property('foo', 'bar')
    assert ctx.foo == 'bar'


def test_add_property_no_cache():
    foo = Mock(return_value=1)
    ctx = Context()
    ctx.add_property('foo', foo, cached=False)
    assert ctx.foo == 1
    assert ctx.foo == 1
    foo.assert_has_calls([call(ctx), call(ctx)])


def test_mapper_add_ctx_property():
    foo = Mock(return_value=1)
    mapper = Mapper()
    mapper.add_ctx_property('foo', foo)

    def resource(request, ctx):
        ctx.foo
        ctx.foo
        return ok()

    mapper.add('/', resource)
    res = mapper(Request({'REQUEST_METHOD': 'GET', 'PATH_INFO': '/'}))
    assert res.code == 200
    assert foo.call_count == 1


def test_mapper_add_ctx_property_duplicate():
    mapper = Mapper()
    mapper.add_ctx_property('foo', 1)
    assert_raises(InvalidArgumentError, mapper.add_ctx_property, 'foo', 2)


def test_callbacks():
    ctx = Context()
    cb = Mock()
    ctx.add_callback('enter', cb)
    ctx._run_callbacks('enter', 1, 2)
    assert cb.mock_calls == [call(1, 2)]


def test_invalid_callback():
    ctx = Context()
    assert_raises(KeyError, ctx.add_callback, 'invalid', lambda: None)



# encoding: utf-8
import time
from datetime import datetime, timedelta
from wsgiref.util import setup_testing_defaults

from mock import patch
from pytest import raises as assert_raises
from rhino.response import Entity, Response, \
        response, ok, created, no_content, redirect, \
        datetime_to_httpdate
from rhino.request import Request


def wsgi_response(response, environ=None):
    if environ is None:
        environ = {}
    rv = []

    def start_response(*args):
        rv.extend(args)

    app_iter = response(environ, start_response)
    body = ''.join(app_iter)
    if hasattr(app_iter, 'close'):
        app_iter.close()
    status, headers = rv
    return status, headers, body


def test_defaults():
    res = Response(200)
    assert res.code == 200
    assert res.status == "200 OK"
    assert res.headers.items() == []
    assert res.body == ''


def test_finalize():
    status, headers, body = wsgi_response(Response(200))
    assert status == "200 OK"
    assert headers == [
        ('Content-Type', 'text/plain; charset=utf-8'),
        ('Content-Length', '0')
    ]
    assert body == ''


def test_finalize_head():
    status, headers, body = wsgi_response(
            Response(200, body='test'), {'REQUEST_METHOD': 'HEAD'})
    assert status == "200 OK"
    assert headers == [
        ('Content-Type', 'text/plain; charset=utf-8'),
        ('Content-Length', '4')
    ]
    assert body == ''


def test_finalize_lazy():
    body = lambda: 'test'
    status, headers, body = wsgi_response(Response(200, body=body))
    assert status == "200 OK"
    assert headers == [
        ('Content-Type', 'text/plain; charset=utf-8'),
        ('Content-Length', '4')
    ]
    assert body == 'test'


def test_finalize_lazy_unicode():
    body = lambda: u'★'
    status, headers, body = wsgi_response(Response(200, body=body))
    assert status == "200 OK"
    assert headers == [
        ('Content-Type', 'text/plain; charset=utf-8'),
        ('Content-Length', str(len(u'★'.encode('utf-8'))))
    ]
    assert body == u'★'.encode('utf-8')


def test_finalize_lazy_invalid():
    body = lambda: None
    assert_raises(TypeError, wsgi_response, Response(200, body=body))


def test_entity():
    entity = Entity('test', content_type='text/html')
    assert entity.body == 'test'
    assert entity.headers.items() == [('Content-Type', 'text/html')]


def test_response_from_entity():
    entity = Entity('test', content_type='text/html')

    status, headers, body = wsgi_response(Response(200, body=entity))
    assert headers == [
        ('Content-Type', 'text/html'),
        ('Content-Length', '4'),
    ]

    status, headers, body = wsgi_response(response(200, body=entity,
                                content_type='text/plain'))
    assert headers == [
        ('Content-Type', 'text/plain'),
        ('Content-Length', '4'),
    ]


def test_headers():
    environ = {}
    setup_testing_defaults(environ)
    res = Response(200, [('X-Foo', u'Smørebrød'), ('Location', u'/☃')], u'☃')
    status, headers, body = wsgi_response(res, environ)
    assert status == "200 OK"
    assert headers == [
        ('X-Foo', u'Smørebrød'.encode('latin-1')),
        ('Content-Type', 'text/plain; charset=utf-8'),
        ('Content-Length', str(len(u'☃'.encode('utf-8')))),
        ('Location', 'http://127.0.0.1/%E2%98%83'),
    ]
    assert body == u'☃'.encode('utf-8')


def test_location_is_absolute():
    environ = {}
    setup_testing_defaults(environ)
    res = Response(302, [('Location', 'http://foo.com/bar;quux?a=b&c=1#x')], '')
    status, headers, body = wsgi_response(res, environ)
    assert headers == [
        ('Content-Type', 'text/plain; charset=utf-8'),
        ('Content-Length', '0'),
        ('Location', 'http://foo.com/bar;quux?a=b&c=1#x'),
    ]


def test_close_callbacks():
    class callback(object):
        def __init__(self):
            self.called = False

        def __call__(self):
            self.called = True

    environ = {}
    setup_testing_defaults(environ)

    cb = callback()
    res = response(200, body='test')
    res.add_callback(cb)
    status, headers, body = wsgi_response(res, environ)
    assert body == 'test'
    assert cb.called


def test_set_cookie_defaults():
    res = response(200)
    res.set_cookie('foo', 'bar')
    assert res.headers.get_all('Set-Cookie') == ['foo=bar; Path=/']


def test_set_cookie_all():
    res = response(200)
    res.set_cookie(
        key='a',
        value=u'Smørebrød',
        max_age=600,
        path='/test',
        domain='example.net',
        secure=True,
        httponly=True,
        expires=datetime(2012, 12, 8, 0, 0, 0),
    )
    cookies = res.headers.get_all('Set-Cookie')
    assert len(cookies) == 1
    # The 'expires' date format in the stdlib's cookie module was fixed
    # in a 2.7 point release.
    assert cookies[0] in [
        'a="Sm\\303\\270rebr\\303\\270d"; Domain=example.net; expires=Sat, 08-Dec-2012 00:00:00 GMT; httponly; Max-Age=600; Path=/test; secure',
        'a="Sm\\303\\270rebr\\303\\270d"; Domain=example.net; expires=Sat, 08 Dec 2012 00:00:00 GMT; httponly; Max-Age=600; Path=/test; secure',
    ]


def test_set_cookie_timedelta():
    res = response(200)
    res.set_cookie('foo', 'bar', max_age=timedelta(minutes=5))
    assert res.headers.get_all('Set-Cookie') == ['foo=bar; Max-Age=300; Path=/']

    res = response(200)
    with patch.object(time, 'time') as mock_time:
        mock_time.return_value = 1.0
        res.set_cookie('foo', 'bar', expires=timedelta(minutes=5))
    cookies = res.headers.get_all('Set-Cookie')
    assert len(cookies) == 1
    # The 'expires' date format in the stdlib's cookie module was fixed
    # in a 2.7 point release.
    assert cookies[0] in [
        'foo=bar; expires=Thu, 01-Jan-1970 00:05:01 GMT; Path=/',
        'foo=bar; expires=Thu, 01 Jan 1970 00:05:01 GMT; Path=/',
    ]

def test_delete_cookie():
    res = response(200)
    res.delete_cookie('x', path='/', domain='example.net')
    cookies = res.headers.get_all('Set-Cookie')
    assert len(cookies) == 1
    # The 'expires' date format in the stdlib's cookie module was fixed
    # in a 2.7 point release.
    assert cookies[0] in [
        'x=; Domain=example.net; expires=Thu, 01-Jan-1970 00:00:00 GMT; Max-Age=0; Path=/',
        'x=; Domain=example.net; expires=Thu, 01 Jan 1970 00:00:00 GMT; Max-Age=0; Path=/',
    ]


def test_conditional_to_etag():
    orig = response(200, 'test', etag='0xdecafbad')

    res = orig.conditional_to(Request({'HTTP_IF_NONE_MATCH': '"0xdecafbad"'}))
    assert res.code == 304
    assert res.body == ''
    assert res.headers['Etag'] == '"0xdecafbad"'

    res = orig.conditional_to(Request({'HTTP_IF_NONE_MATCH': '"0xabad1dea"'}))
    assert res is orig


def test_conditional_to_etag_404():
    orig = response(404, 'not found', etag='0xdecafbad')

    res = orig.conditional_to(Request({'HTTP_IF_NONE_MATCH': '"0xdecafbad"'}))
    assert res is orig


def test_conditional_to_last_modified():
    dt = datetime(2014, 4, 1, 12, 0)
    dt2 = dt - timedelta(seconds=1)
    orig = response(200, 'test', last_modified=dt)

    res = orig.conditional_to(Request({'HTTP_IF_MODIFIED_SINCE': datetime_to_httpdate(dt)}))
    assert res.code == 304
    assert res.body == ''
    assert res.headers['Last-Modified'] is None

    res = orig.conditional_to(Request({'HTTP_IF_MODIFIED_SINCE': datetime_to_httpdate(dt2)}))
    assert res is orig

    res = orig.conditional_to(Request({'HTTP_IF_MODIFIED_SINCE': 'invalid'}))
    assert res is orig


def test_etag_and_last_modified():
    dt = datetime(2014, 4, 1, 12, 0)
    dt2 = dt - timedelta(seconds=1)
    orig = response(200, 'test', last_modified=dt, etag='0xdecafbad')

    res = orig.conditional_to(Request({'HTTP_IF_NONE_MATCH': '"0xdecafbad"'}))
    assert res.code == 304

    res = orig.conditional_to(Request({'HTTP_IF_MODIFIED_SINCE': datetime_to_httpdate(dt)}))
    assert res.code == 304

    res = orig.conditional_to(Request({'HTTP_IF_NONE_MATCH': '"0xdecafbad"', 'HTTP_IF_MODIFIED_SINCE': datetime_to_httpdate(dt)}))
    assert res.code == 304

    # FIXME Should matching ETag take precedence (see comment below)?
    res = orig.conditional_to(Request({'HTTP_IF_NONE_MATCH': '"0xabad1dea"', 'HTTP_IF_MODIFIED_SINCE': datetime_to_httpdate(dt)}))
    assert res is orig

    res = orig.conditional_to(Request({'HTTP_IF_NONE_MATCH': '"0xabad1dea"', 'HTTP_IF_MODIFIED_SINCE': datetime_to_httpdate(dt2)}))
    assert res is orig

    # FIXME It makes sense that ETag should take precedence here, but I think the spec says both conditions must match if both
    # headers are present.
    #res = orig.conditional_to(Request({'HTTP_IF_NONE_MATCH': '"0xdecafbad"', 'HTTP_IF_MODIFIED_SINCE': 'invalid'}))
    #assert res.code == 304


def test_response_defaults():
    res = response(200)
    assert res.code == 200
    assert res.body == ''


def test_response_last_modified():
    dt = datetime.utcnow()
    res = response(200, last_modified=dt)
    assert res.headers['Last-Modified'] == datetime_to_httpdate(dt)

    res = response(200, last_modified=None)
    assert 'Last-Modified' not in res.headers


def test_response_etag():
    res = response(200, etag='foo')
    assert res.headers['ETag'] == '"foo"'

    res = response(200, etag='"foo"')
    assert res.headers['ETag'] == '"foo"'

    res = response(200, etag=None)
    assert 'Etag' not in res.headers

    assert_raises(TypeError, response, 200, etag=123)


def test_response_expires():
    dt = datetime.utcnow() + timedelta(seconds=60)

    res = response(200, expires=60)
    assert res.headers['Expires'] == datetime_to_httpdate(dt)

    res = response(200, expires=timedelta(seconds=60))
    assert res.headers['Expires'] == datetime_to_httpdate(dt)

    res = response(200, expires=dt)
    assert res.headers['Expires'] == datetime_to_httpdate(dt)


def test_ok():
    res = ok()
    assert res.code == 200

    res = ok('hi', code=201, x_foo='bar', etag='x')
    assert res.code == 201
    assert res.body == 'hi'
    assert res.headers['X-Foo'] == 'bar'
    assert res.headers['Etag'] =='"x"'

    assert_raises(ValueError, ok, code=199)
    assert_raises(ValueError, ok, code=300)


def test_created():
    res = created()
    assert res.code == 201


def test_no_content():
    res = no_content()
    assert res.code == 204
    assert_raises(TypeError, no_content, body='test')


def test_redirect():
    assert_raises(TypeError, redirect)

    res = redirect('/')
    assert res.code == 302
    assert res.headers.items() == [('Location', '/')]

    res = redirect('/foo', code=301, x_foo='bar')
    assert res.code == 301
    assert res.headers['Location'] == '/foo'
    assert res.headers['X-Foo'] == 'bar'

    assert_raises(ValueError, redirect, '/', code=299)
    assert_raises(ValueError, redirect, '/', code=400)


#def test_conditional_lazy_body():
#    class Resource(object):
#        def __init__(self):
#            self.body_called = False
#
#        def __call__(self, request):
#            def body():
#                self.body_called = True
#                return 'test'
#            return ok(body, etag="0xdecafbad")
#
#    resource = Resource()
#    app = Mapper()
#    app.add('/', resource)
#    client = TestClient(app.wsgi)
#
#    res = client.get('/')
#    assert res.headers['Content-Length'] == '4'
#    assert res.body == 'test'
#    assert resource.body_called
#
#    resource.body_called = False
#    res = client.get('/', if_none_match='"0xdecafbad"')
#    assert res.status == "304 Not Modified"
#    assert res.body == ''
#    assert not resource.body_called
#
#def test_conditional_lazy_body_iter():
#    class Resource(object):
#        def __init__(self):
#            self.body_called = False
#
#        def __call__(self, request):
#            def body():
#                self.body_called = True
#                yield 'foo'
#                yield 'bar'
#            return ok(body(), etag="0xdecafbad")
#
#    resource = Resource()
#    app = Mapper()
#    app.add('/', resource)
#    client = TestClient(app.wsgi)
#
#    res = client.get('/')
#    assert 'Content-Length' not in res.headers
#    assert res.body == 'foobar'
#    assert resource.body_called
#
#    resource.body_called = False
#    res = client.get('/', if_none_match='"0xdecafbad"')
#    assert res.status == "304 Not Modified"
#    assert res.body == ''
#    assert not resource.body_called



from datetime import datetime, timedelta

from pytest import raises as assert_raises
from rhino.http import datetime_to_httpdate, match_etag, total_seconds, \
        cache_control


def test_datetime_to_httpdate():
    assert datetime_to_httpdate(1) == 'Thu, 01 Jan 1970 00:00:01 GMT'
    assert datetime_to_httpdate(datetime(1970, 1, 2)) == 'Fri, 02 Jan 1970 00:00:00 GMT'


def test_match_etag():
    assert not match_etag(None, '*')
    assert not match_etag(None, '"abc"')
    assert not match_etag(None, '"abc"', weak=True)
    assert not match_etag('"abc"', '"def"')
    assert not match_etag('"abc"', '"def", w/"abc"')

    assert match_etag('"abc"', '"def", w/"abc"', weak=True)
    assert match_etag('"abc"', '*')
    assert match_etag('w/"abc"', '*')
    assert match_etag('"abc"', '"abc"')

    assert not match_etag('w/"abc"', '"abc"')
    assert not match_etag('"abc"', 'w/"abc"')
    assert not match_etag('"abc"', 'x/"abc"')

    assert match_etag('w/"abc"', '"abc"', weak=True)
    assert match_etag('"abc"', 'w/"abc"', weak=True)
    assert match_etag('"abc"', '"abc", w/"abc"')


def test_match_malformed_etag():
    assert_raises(ValueError, match_etag, 'xxx', '"xxx"')


def test_total_seconds():
    td = timedelta(days=2.5, seconds=.1, microseconds=54)
    assert total_seconds(td) == 216000.100054


def test_cache_control():
    assert_raises(ValueError, cache_control, public=1, private=1)
    assert cache_control(private=True) == 'private'
    assert cache_control(max_age=60, public=True, s_maxage=120,
        must_revalidate=True, proxy_revalidate=True, no_cache=True,
        no_store=True) == 'public, max-age=60, s-maxage=120, no-cache, no-store, must-revalidate, proxy-revalidate'
    assert cache_control(max_age=timedelta(hours=2), s_maxage=timedelta(hours=1)) == 'max-age=7200, s-maxage=3600'



import os
import shutil
import tempfile

import pytest
from rhino.test import TestClient

from examples.movie_db import model
from examples.movie_db.web import get_app


@pytest.fixture(scope='module')
def client(request):
    tmpdir = tempfile.mkdtemp(prefix='rhino_test')
    db_path = os.path.abspath(os.path.join(tmpdir, 'movies.db'))
    db_url = 'sqlite:///%s' % db_path
    model.init_db(db_url)

    def finalize():
        shutil.rmtree(tmpdir)

    request.addfinalizer(finalize)
    app = get_app(db_url)
    return TestClient(app.wsgi)


def test_index(client):
    res = client.get('/')
    assert res.headers['Content-Type'] == 'text/html; charset=utf-8'
    assert "Movie Database" in res.body


def test_movie(client):
    res = client.get('/movies/1')
    assert "Star Wars" in res.body


def test_actor(client):
    res = client.get('/actors/1')
    assert "Mark Hamill" in res.body



"""
This module contains a SessionProperty class that can be used to add a
SQLALchemy Session object to the context.

This extension requires the SQLAlchemy module to be installed::

    $ pip install sqlalchemy

The session is lazily initialized and closed at the end of every request.

Example usage::

    from rhino import Mapper, get
    from rhino.ext.sqlalchemy import SessionProperty

    from models import Movie  # module containing SQLAlchemy ORM classes

    app = rhino.Mapper()
    app.add_ctx_property('db', SessionProperty('sqlite:///db.sqlite'))

    @get
    def index(request, ctx):
        movies = ctx.db.query(Movie).all()
        # ...

    app.add('/', index)

"""
from __future__ import absolute_import

from sqlalchemy import create_engine
from sqlalchemy.orm import Session


class SessionProperty(object):
    def __init__(self, url=None, delay_close=False, **session_args):
        if url is not None:
            session_args['bind'] = create_engine(url)
        self.session_args = session_args
        self.delay_close = delay_close

    def __call__(self, ctx):
        session = Session(**self.session_args)
        if self.delay_close:
            ctx.add_callback('close', session.close)
        else:
            ctx.add_callback('teardown', session.close)
        return session



import os, re, cgi, stat, time
import html

wikiname_re = re.compile('((?<![a-z\d])([A-Z][a-z]+([A-Z][a-z]+|\d+)+))')

def wikify(text):
    return wikiname_re.sub(r'<a href="/\1">\1</a>', cgi.escape(text))

class Storage(object):
    def __init__(self, directory):
        self.directory = directory

    def get_current(self, title, create=False):
        current_file = os.path.join(self.directory, title, 'current')
        if os.path.exists(current_file):
            with open(current_file) as f:
                version_file = f.read()
            version = os.path.basename(version_file)
            version = version.split('-')[1]
        elif not create:
            raise KeyError(title)
        else:
            os.mkdir(os.path.join(self.directory, title))
            version_file = '' #os.path.join(self.directory, title, 'version-1')
            version = '0'

        return current_file, version_file, version

    def update_current(self, title, version_file):
        current_file = os.path.join(self.directory, title, 'current')
        if os.path.exists(current_file):
            os.remove(current_file)

        # move the current version
        with open(current_file, 'w') as f:
            f.write(version_file)

    def store_page(self, title, content):
        current, version_file, version = self.get_current(title,
            create=True)

        # new version
        new_version = int(version) + 1
        version_file = os.path.join(self.directory, title,
            'version-%d' % new_version)

        with open(version_file, 'w') as f:
            f.write(content)

        self.update_current(title, version_file)
        return new_version

    def retrieve_page(self, title, version=None):
        dir = os.path.join(self.directory, title)
        if not os.path.exists(dir):
            raise KeyError(title)

        if version is None:
            current_file, version_file, version = self.get_current(title)
        else:
            version_file = os.path.join(dir, 'version-%s'%version)
            if not os.path.exists(version_file):
                raise KeyError(version_file)

        with open(version_file) as f:
            content = f.read()
        return content
        
    def revert_page(self, title, version):
        current_file = os.path.join(self.directory, title, 'current')
        if not os.path.exists(current_file):
            raise KeyError(title)

        version = os.path.join(self.directory, title, 'version-%s'%version)
        with open(current_file, 'w') as f:
            f.write(version)

    def list_page_versions(self, title):
        dir = os.path.join(self.directory, title)
        if not os.path.exists(dir):
            raise KeyError(title)

        # determine the current version
        current_file, version_file, version = self.get_current(title,
            create=True)

        versions = []
        for i in range(int(version)):
            path = os.path.join(dir, 'version-%d'%(i+1))
            mtime = os.stat(path)[stat.ST_MTIME]
            versions.append((mtime, str(i+1)))
        versions.sort()
        return [(v, t) for t, v in versions]

    def render_page(self, name):
        try:
            content = self.retrieve_page(name)
        except KeyError:
            content = 'Page does not exist: edit to create it.'
        h = html.HTML()
        h.h1(name)
        h.pre(wikify(content), escape=False)
        h.a('edit this page', href='/%s/edit' % name)
        h.br
        h.a('page history', href='/%s/history' % name)
        return str(h)

    def render_edit_form(self, name):
        try:
            content = self.retrieve_page(name)
        except KeyError:
            content = 'Page does not exist: edit to create it.'
        h = html.HTML()
        h.h1('Editing ' + name)
        f = h.form(action='/%s/edit' % name, method='POST')
        f.textarea(content, rows='10', cols='60', name='content')
        f.br
        f.input(type='submit', name='submit', value='Save Changes')
        f.input(type='submit', name='cancel', value='Cancel')
        return str(h)

    def render_history_form(self, name):
        h = html.HTML()
        h.h1('History For ' + name)
        h.p('Select version to revert to.')
        f = h.form(action='/%s/history' % name, method='POST')
        for version, ts in self.list_page_versions(name):
            l = f.li(time.asctime(time.localtime(ts)))
            l.input(type='radio', value=str(version), name='version')
        f.br
        f.input(type='submit', name='submit', value='Revert To Selected')
        f.input(type='submit', name='cancel', value='Cancel')
        return str(h)





from __future__ import absolute_import

from cgi import escape

from .http import status_codes
from .response import Response

# Default HTML error page, inspired by Django.
html_template = '''<!DOCTYPE html>
<html>
  <head>
    <title>%(status)s</title>
    <style type="text/css">
      html { font-family: sans-serif; font-size: small; color: #333; }
      html, body { margin: 0; padding: 0; }
      body > * { margin: 0; padding: 10px; }
      h1 { font-size: 180%%; font-weight: normal; background: wheat; border-bottom: 1px solid #ccc; }
      h1 small { font-size: 60%%; color: #777; }
      p { background: #eee; border-bottom: 1px solid #ccc; }
      p + p { background: #fff; border: 0; }
    </style>
  </head>
  <body>
    <h1>%(status)s <small>(%(code)s)</small></h1>
    <p>%(message)s</p>
    <p>%(details)s</p>
  </body>
</html>
'''

class HTTPException(Exception):
    """Base class for HTTP Exceptions

    Class variables:

    code
        The status code (int)
    message
        If present, an HTML error page will be sent that contains this message.
        The contents of `message` will be HTML-escaped.

    Instance properties:

    response
        The `rhino.Response` instance that will be sent to the client.

    The constructor takes one argument, an optional message that will replace
    the default message. Subclasses can override the constructor to require
    error-specific arguments.
    """
    code = 500
    message = None
    details = None  # TODO for displaying structured error data in debug mode

    def __init__(self, message=None):
        if message is None:
            message = self.message
        if message is not None:
            body = html_template % {
                'code': self.code,
                'status': status_codes.get(self.code, "Unknown"),
                'message': escape(message),
                'details': self.details or '',
            }
            headers = [('Content-Type', 'text/html')]
        else:
            body, headers = '', []
        self.response = Response(self.code, body=body, headers=headers)


class Redirection(HTTPException):
    """Base class for redirections (3xx)."""
    pass

class ClientError(HTTPException):
    """Base class for client errors (4xx)."""
    pass

class ServerError(HTTPException):
    """Base class for server errors (5xx)."""
    pass


class MovedPermanently(Redirection):
    """301 Moved Permanently.

    Required arguments:

    location
        The value for the Location header.
    """
    code = 301

    def __init__(self, location):
        super(MovedPermanently, self).__init__()
        self.response.headers['Location'] = location


class Found(Redirection):
    """302 Found.

    Required arguments:

    location
        The value for the Location header.
    """
    code = 302

    def __init__(self, location):
        super(Found, self).__init__()
        self.response.headers['Location'] = location


class SeeOther(Redirection):
    """303 See Other.

    Required arguments:

    location
        The value for the Location header.
    """
    code = 303

    def __init__(self, location):
        super(SeeOther, self).__init__()
        self.response.headers['Location'] = location


class TemporaryRedirect(Redirection):
    """307 Temporary Redirect.

    Required arguments:

    location
        The value for the Location header.
    """
    code = 307

    def __init__(self, location):
        super(TemporaryRedirect, self).__init__()
        self.response.headers['Location'] = location


class BadRequest(ClientError):
    """400 Bad Request."""
    code = 400
    message = 'The server could not understand the request.'


class Unauthorized(ClientError):
    """401 Unauthorized.

    Required arguments:

    scheme
        The authentication scheme to use, e.g. 'Basic'.

    `**params`
        Parameters for the WWW-Authenticate header, e.g. realm="my website".
    """
    code = 401

    def __init__(self, scheme, **params):
        super(Unauthorized, self).__init__()
        param_str = ', '.join(['%s="%s"' % (k, v)
                               for k, v in sorted(params.items())])
        www_authenticate = "%s %s" % (scheme, param_str)
        self.response.headers['WWW-Authenticate'] = www_authenticate


class Forbidden(ClientError):
    """403 Forbidden."""
    code = 403
    message = 'The server is refusing to fulfill the request.'


class NotFound(ClientError):
    """404 NotFound."""
    code = 404
    message = 'The requested resource could not be found.'


class MethodNotAllowed(ClientError):
    """404 Method Not Allowed.

    Required arguments:

    allow
        The value for the 'Allow' header (A list of comma-separated HTTP
        method names).
    """
    code = 405
    message = 'The request method is not allowed for this resource.'

    def __init__(self, allow):
        super(MethodNotAllowed, self).__init__()
        self.response.headers['Allow'] = allow


class NotAcceptable(ClientError):
    """406 Not Acceptable."""
    code = 406
    message = 'The resource is not capable of generating a response entity in an acceptable format.'


class Gone(ClientError):
    """410 Gone."""
    code = 410
    message = 'The requested resource is no longer available.'
    details = """
        <q style="font-style: italic; quotes: none;">Embracing HTTP error code
        410 means embracing the impermanence of all things.</q> &mdash; Mark
        Pilgrim
    """

class UnsupportedMediaType(ClientError):
    """415 Unsupported Media Type."""
    code = 415
    message = 'The request entity is in a format that is not supported by this resource.'


class InternalServerError(ServerError):
    """500 Internal Server Error."""
    code = 500
    message = 'The server encountered an error while processing the request.'



from __future__ import absolute_import

import re
import sys
import urllib

from .errors import HTTPException, InternalServerError, NotFound
from .request import Request
from .response import Response
from .resource import Resource
from .util import call_with_ctx

# template2regex function taken from Joe Gregorio's wsgidispatcher.py
# (https://code.google.com/p/robaccia/) with minor modifications.

class MapperException(Exception): pass
class InvalidArgumentError(MapperException): pass
class InvalidTemplateError(MapperException): pass

template_splitter = re.compile("([\[\]\{\}\|])")
string_types = (str, unicode)

DEFAULT_RANGES = {
    'word': r'\w+',
    'alpha': r'[a-zA-Z]+',
    'digits': r'\d+',
    'alnum': r'[a-zA-Z0-9]+',
    'segment': r'[^/]+',
    'unreserved': r'[a-zA-Z\d\-\.\_\~]+',
    'any': r'.+'
}

# The conversion done in template2regex can be in one of two states, either
# handling a path, or it can be inside a {} template.
# The conversion done in template2path can additionaly be skipping an
# optional [] block.
S_PATH = 0
S_TEMPLATE = 1
S_SKIP = 2


def template2regex(template, ranges=None):
    """Convert a URL template to a regular expression.

    Converts a template, such as /{name}/ to a regular expression, e.g.
    /(?P<name>[^/]+)/ and a list of the named parameters found in the template
    (e.g. ['name']). Ranges are given after a colon in a template name to
    indicate a restriction on the characters that can appear there. For
    example, in the template::

        "/user/{id:alpha}"

    The ``id`` must contain only characters from a-zA-Z. Other characters there
    will cause the pattern not to match.

    The ranges parameter is an optional dictionary that maps range names to
    regular expressions. New range names can be added, or old range names can
    be redefined using this parameter.

    Example:

    >>> import rhino.mapper
    >>> rhino.mapper.template2regex("{fred}")
    ('^(?P<fred>[^/]+)$', ['fred'])

    """
    if len(template) and -1 < template.find('|') < len(template) - 1:
        raise InvalidTemplateError("'|' may only appear at the end, found at position %d in %s" % (template.find('|'), template))
    if ranges is None:
        ranges = DEFAULT_RANGES
    anchor = True
    state = S_PATH
    if len(template) and template[-1] == '|':
        anchor = False
    params = []

    bracketdepth = 0
    result = ['^']
    name = ""
    pattern = "[^/]+"
    rangename = None
    for c in template_splitter.split(template):
        if state == S_PATH:
            if len(c) > 1:
                result.append(re.escape(c))
            elif c == '[':
                result.append("(")
                bracketdepth += 1
            elif c == ']':
                bracketdepth -= 1
                if bracketdepth < 0:
                    raise InvalidTemplateError("Mismatched brackets in %s" % template)
                result.append(")?")
            elif c == '{':
                name = ""
                state = S_TEMPLATE
            elif c == '}':
                raise InvalidTemplateError("Mismatched braces in %s" % template)
            elif c == '|':
                pass
            else:
                result.append(re.escape(c))
        else:
            if c == '}':
                if rangename and rangename in ranges:
                    result.append("(?P<%s>%s)" % (name, ranges[rangename]))
                else:
                    result.append("(?P<%s>%s)" % (name, pattern))
                params.append(name)
                state = S_PATH
                rangename = None
            else:
                name = c
                if name.find(":") > -1:
                    name, rangename = name.split(":")
    if bracketdepth != 0:
        raise InvalidTemplateError("Mismatched brackets in %s" % template)
    if state == S_TEMPLATE:
        raise InvalidTemplateError("Mismatched braces in %s" % template)
    if anchor:
        result.append('$')
    return "".join(result), params


def template2path(template, params, ranges=None):
    """Converts a template and a dict of parameters to a path fragment.

    Converts a template, such as /{name}/ and a dictionary of parameter
    values to a URL path (string). Parameter values will be converted to
    strings and escaped.

    Parameter values that are used are converted using `str()` and validated
    against the paremter range. Unused parameters are ignored.

    Any optional ([]) blocks in the template are skipped unless they contain at
    least one parameter and all parameters needed to fill the block (including
    nested blocks) are present in `params`.

    Example:

    >>> import rhino.mapper
    >>> rhino.mapper.template2path("/{name}", {'name': 'fred'})
    '/fred'

    """
    if len(template) and -1 < template.find('|') < len(template) - 1:
        raise InvalidTemplateError("'|' may only appear at the end, found at position %d in %s" % (template.find('|'), template))
    if ranges is None:
        ranges = DEFAULT_RANGES

    # Stack for path components. A new list is added for each '[]' block
    # encountered. When the closing ']' is reached, the last element is
    # removed and either merged into the previous one (we keep the
    # block) or discarded (we skip the block). At the end, this should
    # contain a flat list of strings as its single element.
    stack = [[]]
    pattern = "[^/]+"    # default range
    name = ""            # name of the current parameter
    bracketdepth = 0     # current level of nested brackets
    skip_to_depth = 0    # if > 1, skip until we're back at this bracket level
    state = S_PATH
    rangename = None     # range name for the current parameter
    seen_name = [False]  # have we seen a named param in bracket level (index)?

    for c in template_splitter.split(template):
        if state == S_PATH:
            if len(c) > 1:
                stack[-1].append(c)
            elif c == '[':
                bracketdepth += 1
                stack.append([])
                seen_name.append(False)
            elif c == ']':
                bracketdepth -= 1
                if bracketdepth < 0:
                    raise InvalidTemplateError("Mismatched brackets in %s" % template)
                last_elem = stack.pop()
                if seen_name.pop():
                    stack[-1].extend(last_elem)
                    seen_name[-1] = True
            elif c == '{':
                name = ""
                state = S_TEMPLATE
            elif c == '}':
                raise InvalidTemplateError("Mismatched braces in %s" % template)
            elif c == '|':
                pass
            else:
                stack[-1].append(c)
        elif state == S_SKIP:
            if c == '[':
                bracketdepth += 1
                seen_name.append(False)
            elif c == ']':
                if bracketdepth == skip_to_depth:
                    stack.pop()
                    skip_to_depth = 0
                    state = S_PATH
                bracketdepth -= 1
                seen_name.pop()
        else:  # state == S_TEMPLATE
            if c == '}':
                if name not in params:
                    if bracketdepth:
                        # We're missing a parameter, but it's ok since
                        # we're inside a '[]' block. Skip everything
                        # until we reach the end of the current block.
                        skip_to_depth = bracketdepth
                        state = S_SKIP
                    else:
                        raise InvalidArgumentError("Missing parameter '%s' in %s" % (name, template))
                else:
                    if rangename and rangename in ranges:
                        regex = ranges[rangename]
                    else:
                        regex = pattern
                    value_bytes = unicode(params[name]).encode('utf-8')
                    value = urllib.quote(value_bytes, safe='/:;')
                    if not re.match('^' + regex + '$', value):
                        raise InvalidArgumentError("Value '%s' for parameter '%s' does not match '^%s$' in %s" % (value, name, regex, template))
                    stack[-1].append(value)
                    state = S_PATH
                rangename = None
            else:
                name = c
                if name.find(":") > -1:
                    name, rangename = name.split(":")
                seen_name[bracketdepth] = True

    if bracketdepth != 0:
        raise InvalidTemplateError("Mismatched brackets in %s" % template)
    if state == S_TEMPLATE:
        raise InvalidTemplateError("Mismatched braces in %s" % template)
    # None of these Should Ever Happen [TM]
    if state == S_SKIP:     # pragma: no cover
        raise MapperException("Internal error: end state is S_SKIP")
    if len(stack) > 1:      # pragma: no cover
        raise MapperException("Internal error: stack not empty")
    if len(seen_name) != 1: # pragma: no cover
        raise MapperException("Internal error: seen_name not empty")

    return "".join(stack[0])



_callback_phases = ['enter', 'leave', 'finalize', 'teardown', 'close']

def _callback_dict():
    return dict((k, []) for k in _callback_phases)


class Context(object):
    def __init__(self, request=None):
        self.request = request
        self.__properties = {}
        self.__callbacks = _callback_dict()

    def add_callback(self, phase, fn):
        """Adds a callback to the context.

        The `phase` determines when and if the callback is executed, and which
        positional arguments are passed in:

        'enter'
            Called from `Resource`, after a handler for the current request has
            been resolved, but before the handler is called.

            Arguments: request

        'leave'
            Called from `Resource`, after the handler has returned
            successfully.

            Arguments: request, response

        'finalize'
            Called from `Mapper`, before WSGI response is finalized.

            Arguments: request, response

        'teardown'
            Called from `Mapper`, before control is passed back to the
            WSGI layer.

            Arguments: -

        'close'
            Called when the WSGI server calls `close()` on the response
            iterator.

            Arguments: -

        """
        try:
            self.__callbacks[phase].append(fn)
        except KeyError:
            raise KeyError("Invalid callback phase '%s'. Must be one of %s" % (phase, _callback_phases))

    def _run_callbacks(self, phase, *args):
        for fn in self.__callbacks[phase]:
            fn(*args)

    def add_property(self, name, fn, cached=True):
        if name in self.__properties:
            raise KeyError("Trying to add a property '%s' that already exists on this %s instance." % (name, self.__class__.__name__))
        self.__properties[name] = (fn, cached)
        if not callable(fn):
            setattr(self, name, fn)

    def __getattr__(self, name):
        if name not in self.__properties:
            raise AttributeError("'%s' object has no attribute '%s'"
                    % (self.__class__.__name__, name))
        fn, cached = self.__properties[name]
        value = fn(self)
        if cached:
            setattr(self, name, value)
        return value


class Route(object):
    """
    A Route links a URL template and an optional name to a resource.
    """

    def __init__(self, template, resource, ranges=None, name=None):
        if ranges is None:
            ranges = DEFAULT_RANGES
        if name is not None and any(c in name for c in './'):
            raise InvalidArgumentError("Route name '%s' contains invalid characters ('./')" % name)

        regex, params = template2regex(template, ranges)
        self.regex = re.compile(regex)
        self.params = set(params)
        self.template = template
        self.resource = resource
        self.ranges = ranges
        self.name = name
        self.is_anchored = len(template) and template[-1] != '|'

    def path(self, params):
        """Builds the URL path fragment for this route."""
        return template2path(self.template, params, self.ranges)

    def __call__(self, request, ctx):
        """Try to dispatch a request.

        Returns a `Response` instance, or None if the route does not match.
        """
        path = request.path_info
        match = self.regex.match(path)
        if match:
            request._set_context(route=self)
            environ = request.environ
            match_vars = match.groupdict()
            if match_vars:
                request.routing_args.update(
                        dict((k, v) for k, v in match_vars.items()
                             if v is not None))
            if not self.is_anchored:
                extra_path = path[match.end():]
                script_name = request.script_name + path[:match.end()]
                environ['SCRIPT_NAME'] = script_name.encode('utf-8')
                environ['PATH_INFO'] = extra_path.encode('utf-8')
            return call_with_ctx(self.resource, ctx, request)
        return None


class Mapper(object):
    """
    A Mapper groups resources or other mappers under a common URL root, and
    dispatches incoming requests based on their URL. It also provides the WSGI
    interface for the application.
    """
    default_encoding = None
    default_content_type = None

    # TODO 'root' parameter for manually specifying a URL prefix not reflected
    # in SCRIPT_NAME (e.g. when proxying).
    def __init__(self, ranges=None):
        """Create a new mapper.

        The `ranges` parameter can be used to override or augment the default
        ranges by passing in a dict mapping range names to patterns.
        """
        self.ranges = DEFAULT_RANGES.copy()
        if ranges is not None:
            self.ranges.update(ranges)
        self.routes = []
        self.named_routes = {}
        self._lookup = {}  # index of routes by object ID for faster path(obj)
        self._ctx_properties = {}

    def add(self, template, resource, name=None):
        """Add a route to a resource.

        The optional `name` assigns a name to this route that can be used when
        building URLs. The name must be unique within this Mapper instance.
        """
        # Special case for standalone handler functions
        if hasattr(resource, '_rhino_meta'):
            route = Route(
                    template, Resource(resource), name=name, ranges=self.ranges)
        else:
            route = Route(
                    template, resource, name=name, ranges=self.ranges)
        obj_id = id(resource)
        if obj_id not in self._lookup:
            # It's ok to have multiple routes for the same object id.
            self._lookup[obj_id] = route
        if name is not None:
            if name in self.named_routes:
                raise InvalidArgumentError("A route named '%s' already exists in this %s instance."
                        % (name, self.__class__.__name__))
            self.named_routes[name] = route
        self.routes.append(route)

    def add_ctx_property(self, name, fn, cached=True):
        """Install a context property.

        A context property is a callable that will be called on first access
        of the property named `name` on the context instance. The result
        will be cached unless `cached` is False.

        If the context property is not callable, it will be installed
        as-is, otherwise, it will be called with the context instance as
        first argument.
        """
        if name in self._ctx_properties:
            raise InvalidArgumentError("A context property name '%s' already exists." % name)
        self._ctx_properties[name] = (fn, cached)

    def path(self, target, params):
        """Build a URL path fragment for a resource or route.

        Possible values for `target`:

        A string with no '.'s
            If the string does not contain '.', it will be used to look up
            a named route of this mapper instance and return it's path.

        A string of the form 'a.b', 'a.b.c', etc.
            Follows the route to nested mappers by splitting off consecutive
            segments. Returns the path of the route found by looking up the
            final segment on the last mapper.

        A `Route` instance
            Returns the path for the route.

        A resource that was added previously
            Looks up the first route that points to this resource and
            returns its path.
        """
        if type(target) in string_types:
            if '.' in target:
                # Build path for a dotted route name
                prefix, rest = target.split('.', 1)
                route = self.named_routes[prefix]
                params_copy = params.copy()
                prefix_params = dict((k, params_copy.pop(k))
                                     for k in route.params)
                prefix_path = route.path(prefix_params)
                next_mapper = route.resource
                return prefix_path + next_mapper.path(rest, params_copy)
            else:
                # Build path for a named route
                return self.named_routes[target].path(params)
        elif isinstance(target, Route):
            # Build path for a route instance, used by for build_url('.')
            for route in self.routes:
                if route is target:
                    return route.path(params)
            raise InvalidArgumentError("Route '%s' not found in this %s instance." % (target, self.__class__.__name__))
        else:
            # Build path for route resource instance
            target_id = id(target)
            if target_id in self._lookup:
                return self._lookup[target_id].path(params)
            raise InvalidArgumentError("No Route found for target '%s' in this %s instance." % (target, self.__class__.__name__))

    def wsgi(self, environ, start_response):
        """Implements the mapper's WSGI interface."""
        request = Request(environ)
        ctx = Context(request)
        try:
            response = self(request, ctx)
            response = response.conditional_to(request)
        except HTTPException as e:
            response = e.response
        except Exception:
            self.handle_error(request, sys.exc_info())
            response = InternalServerError().response
        response.add_callback(lambda: ctx._run_callbacks('close'))
        ctx._run_callbacks('finalize', request, response)
        wsgi_response = response(environ, start_response)
        ctx._run_callbacks('teardown')
        return wsgi_response

    # taken and adapted from wsgiref.handlers.BaseHandler
    def handle_error(self, request, exc_info):
        """Log the 'exc_info' tuple in the server log."""
        try:
            from traceback import print_exception
            stream = request.environ.get('wsgi.error', sys.stderr)
            print_exception(exc_info[0], exc_info[1], exc_info[2], None, stream)
            stream.flush()
        finally:
            exc_info = None  # Clear traceback to avoid circular reference

    def __call__(self, request, ctx=None):
        if ctx is None:
            ctx = Context(request=request)
        for name, (fn, cached) in self._ctx_properties.items():
            ctx.add_property(name, fn, cached=cached)
        # TODO here is were we would have to prepend self.root
        request._add_context(root=request.script_name, mapper=self, route=None)
        for route in self.routes:
            response = route(request, ctx)
            if response is not None:
                if not isinstance(response, Response):
                    raise TypeError("Not a rhino.Response instance: %s." % response)
                if self.default_encoding is not None:
                    response.default_encoding = self.default_encoding
                if self.default_content_type is not None:
                    response.default_content_type = self.default_content_type
                return response
        raise NotFound

    def start_server(self, host='localhost', port=9000, app=None):
        """Start a `wsgiref.simple_server` based server to run this mapper."""
        from wsgiref.simple_server import make_server
        if app is None:
            app = self.wsgi
        server = make_server(host, port, app)
        server_addr = "%s:%s" % (server.server_name, server.server_port)
        print "Server listening at http://%s/" % server_addr
        server.serve_forever()



"""
Utilities for testing.
"""
from __future__ import absolute_import

from collections import namedtuple
from StringIO import StringIO
from urllib import urlencode
from wsgiref.headers import Headers
from wsgiref.util import setup_testing_defaults
from wsgiref.validate import validator

from .resource import handler_metadata


def make_handler_metadata(verb=None, view=None, **kw):
    return handler_metadata.create(verb=verb, view=view, **kw)


class wsgi_response(namedtuple('wsgi_response', 'status headers body')):
    @property
    def code(self):
        return int(self.status.split(' ', 1)[0])


class TestClient(object):
    """Wraps a WSGI application under test.

    Provides helpers to pass requests to the application and inspect the
    response.
    """

    def __init__(self, app):
        self.app = app

    def request(self, method, path, environ=None, **kw):
        """Send a request to the application under test.

        The environment will be populated with some default keys. Additional
        keyword arguments will be interpreted as request headers.

        For example, passing x_foo=1 will add a request header "X-Foo: 1".
        """
        if environ is None:
            environ = {}
        # setup_testing_defaults() uses '127.0.0.1', but localhost is easier
        # to type when testing.
        environ.setdefault('SERVER_NAME', 'localhost')
        environ.setdefault('QUERY_STRING', '')  # silence validator warning
        setup_testing_defaults(environ)
        environ['REQUEST_METHOD'] = method
        environ['PATH_INFO'] = path
        for k, v in kw.items():
            key = k.upper()
            if k not in ('content_type', 'content_length'):
                key = 'HTTP_' + key
            environ[key] = str(v)
        start_response_rv = []

        def start_response(status, headers, exc_info=None):
            # TODO handle exc_info != None
            start_response_rv.extend([status, headers])

        wsgi_app = validator(self.app)
        app_iter = wsgi_app(environ, start_response)
        try:
            body = ''.join(app_iter)
        finally:
            if hasattr(app_iter, 'close'):
                app_iter.close()
        statusline, headerlist = start_response_rv
        return wsgi_response(statusline, Headers(headerlist), body)

    def get(self, path_info, environ=None, **kw):
        """Send a GET request to the application."""
        return self.request('GET', path_info, environ, **kw)

    def post(self, path_info, body, content_type=None, environ=None, **kw):
        """Send a POST request to the application.

        If body is a dictionary, it will we submitted as a form with
        content_type='application/x-www-form-urlencoded'. Otherwise,
        the content_type parameter is required."""
        if environ is None:
            environ = {}
        if isinstance(body, dict):
            body = urlencode(body)
            content_type = 'application/x-www-form-urlencoded'
        elif content_type is None:
            raise ValueError("Can't send data without content_type")
        environ.update({
            'CONTENT_TYPE': content_type,
            'CONTENT_LENGTH': str(len(body)),
            'wsgi.input': StringIO(body),
        })
        return self.request('POST', path_info, environ, **kw)

    def put(self, path_info, body, content_type, environ=None, **kw):
        """Send a PUT request to the application."""
        if environ is None:
            environ = {}
        environ.update({
            'CONTENT_TYPE': content_type,
            'CONTENT_LENGTH': str(len(body)),
            'wsgi.input': StringIO(body),
        })
        return self.request('PUT', path_info, environ, **kw)

    def head(self, path_info, environ=None, **kw):
        """Send a HEAD request to the application."""
        return self.request('HEAD', path_info, environ, **kw)

    def options(self, path_info, environ=None, **kw):
        """Send an OPTIONS request to the application."""
        return self.request('OPTIONS', path_info, environ, **kw)

    def delete(self, path_info, environ=None, **kw):
        """Send a DELETE request to the application."""
        return self.request('DELETE', path_info, environ, **kw)



import jinja2
from rhino.mapper import Context
from rhino.request import Request
from rhino.ext.jinja2 import JinjaRenderer
from rhino.test import TestClient


def test_default_loader():
    renderer = JinjaRenderer('./templates')
    assert isinstance(renderer.env.loader, jinja2.FileSystemLoader)
    assert renderer.env.loader.searchpath == ['./templates']


def test_renderer():
    renderer = JinjaRenderer(loader=jinja2.DictLoader({'test': '{{ foo }}'}))
    ctx = Context()
    ctx.request = Request({})
    ctx.add_property('render_template', renderer)
    entity = ctx.render_template('test', foo='<script>')
    assert entity.body == '&lt;script&gt;'
    assert entity.headers['Content-Type'] == 'text/html; charset=utf-8'



import functools

from pytest import raises as assert_raises

from rhino.util import dual_use_decorator, dual_use_decorator_method, \
        get_args, sse_event


@dual_use_decorator
def decorator_function(*args, **kw):
    """decorator function"""
    decorator_args = (args, kw)
    def decorator(fn):
        @functools.wraps(fn)
        def wrapper(*args, **kw):
            return {'func_args': (args, kw),
                    'decorator_args': decorator_args}
        return wrapper
    return decorator


class MyClass(object):
    @dual_use_decorator_method
    def decorator_method(self, *args, **kw):
        """decorator method"""
        decorator_args = (args, kw)
        def decorator(fn):
            @functools.wraps(fn)
            def wrapper(*args, **kw):
                return {'func_args': (args, kw),
                        'decorator_args': decorator_args}
            return wrapper
        return decorator


def test_docstrings():
    assert decorator_function.__doc__ == 'decorator function'
    assert MyClass.decorator_method.__doc__ == 'decorator method'


def test_dual_use_decorator():
    @decorator_function
    def foo():
        """foo function"""
        pass

    assert foo(1, a=2) == {
        'func_args': ((1,), {'a': 2}),
        'decorator_args': (tuple(), {}),
    }
    assert foo.__doc__ == 'foo function'


def test_dual_use_decorator_with_args():
    @decorator_function(3, b=4)
    def foo():
        """foo function"""
        pass

    assert foo(1, a=2) == {
        'func_args': ((1,), {'a': 2}),
        'decorator_args': ((3,), {'b': 4}),
    }
    assert foo.__doc__ == 'foo function'


def test_dual_use_decorator_method():
    obj = MyClass()
    @obj.decorator_method
    def foo():
        """foo function"""
        pass

    assert foo(1, a=2) == {
        'func_args': ((1,), {'a': 2}),
        'decorator_args': (tuple(), {}),
    }
    assert foo.__doc__ == 'foo function'


def test_dual_use_decorator_method_with_args():
    obj = MyClass()
    @obj.decorator_method(3, b=4)
    def foo():
        """foo function"""
        pass

    assert foo(1, a=2) == {
        'func_args': ((1,), {'a': 2}),
        'decorator_args': ((3,), {'b': 4}),
    }
    assert foo.__doc__ == 'foo function'


def test_get_args():
    class Foo(object):
        def __init__(self, a, b):
            pass

        def __call__(self, e, f):
            pass

        def foo(self, g, h):
            pass

        @classmethod
        def bar(cls, i, j):
            pass

        @staticmethod
        def baz(k, l):
            pass

    foo = Foo(None, None)

    assert get_args(lambda x: None) == ['x']

    assert get_args(Foo) == ['a', 'b']
    assert get_args(Foo.foo) == ['g', 'h']
    assert get_args(Foo.bar) == ['i', 'j']
    assert get_args(Foo.baz) == ['k', 'l']

    assert get_args(foo) == ['e', 'f']
    assert get_args(foo.foo) == ['g', 'h']
    assert get_args(foo.bar) == ['i', 'j']
    assert get_args(foo.baz) == ['k', 'l']

    assert_raises(TypeError, get_args, None)


def test_sse_event():
    assert sse_event('test', 'foo\nbar') == \
            'event: test\ndata: foo\ndata: bar\n\n'
    assert sse_event(comment='a\nb\n') == \
            ': a\n: b\n: \n\n'
    assert sse_event(
            event='test', data='foo', id='id', retry=12, comment='hi') == \
            ': hi\nevent: test\nid: id\nretry: 12\ndata: foo\n\n'

def test_sse_event_minimal():
    assert sse_event(comment='') == ': \n\n'


def test_sse_event_empty():
    assert_raises(TypeError, sse_event)



from __future__ import absolute_import

import cgi
import urllib
import urlparse
from Cookie import SimpleCookie
from wsgiref.util import request_uri, application_uri

from .urls import request_context, build_url


class RequestHeaders(object):
    """A dictionary-like object to access request headers.

    Keys are case-insensitive. Accessing a header that is not present
    returns None instead of raising KeyError.
    """

    def __init__(self, environ):
        self.environ = environ

    @staticmethod
    def _key(name):
        key = name.upper().replace('-', '_')
        if key in ('CONTENT_TYPE', 'CONTENT_LENGTH'):
            return key
        return 'HTTP_' + key

    @staticmethod
    def _name(key):
        if key[:5] == 'HTTP_':
            key = key[5:]
        return key.replace('_', '-').title()

    def _keys(self):
        return [k for k in self.environ
                if k[:5] == 'HTTP_' or k in ('CONTENT_TYPE', 'CONTENT_LENGTH')]

    def __len__(self):
        return len(self._keys())

    def __contains__(self, name):
        return self._key(name) in self.environ

    def get(self, name, default=None):
        value = self.environ.get(self._key(name), default)
        return default if value is default else value.decode('latin-1')

    def __getitem__(self, name):
        return self.get(name)

    def keys(self):
        return [self._name(k) for k in self._keys()]

    def values(self):
        return [self.environ[k].decode('latin-1') for k in self._keys()]

    def items(self):
        return [(self._name(k), self.environ[k].decode('latin-1'))
                for k in self._keys()]


_default = object()  # unique canary value, used by QueryDict.__getitem__


class QueryDict(object):
    """A dictionary-like object to access query parameters.

    Accessing a key returns the first value for that key.
    The keys(), values() and items() methods will return a key/value multiple
    times, if it is present multiple times in the query string.

    The get_all() method can be used to return all values for a given key.
    """

    def __init__(self, items):
        self._items = items

    def __len__(self):
        return len(self._items)

    def __contains__(self, key):
        try:
            self[key]
        except KeyError:
            return False
        return True

    def __getitem__(self, key):
        v = self.get(key, _default)
        if v is _default:
            raise KeyError(key)
        return v

    def get_all(self, key):
        """Return a list of values for the given key."""
        return [v for k, v in self._items if k == key]

    def get(self, key, default=None):
        for k, v in self._items:
            if k == key:
                return v
        return default

    def keys(self):
        return [k for k, v in self._items]

    def values(self):
        return [v for k, v in self._items]

    def items(self):
        return self._items[:]


class Request(object):
    """Represents an HTTP request built from a WSGI environment."""

    def __init__(self, environ):
        self.environ = environ
        self.headers = RequestHeaders(environ)
        self._url = None
        self._body = None
        self._form = None
        self._query = None
        self._cookies = None
        self._context = []
        self._application_uri = None

    def _add_context(self, **kw):
        self._context.append(request_context(**kw))

    def _set_context(self, **kw):
        if not self._context:  # pragma: no cover
            raise RuntimeError("No routing context present.")
        self._context[-1] = self._context[-1]._replace(**kw)

    def url_for(*args, **kw):
        """Build the URL for a target.

        Special keyword arguments:

        `_query`
            Append query string (dict or list of tuples)

        `_relative`
            When True, build a relative URL (default: False)

        All other keyword arguments are treated as parameters for the URL
        template.

        The target can be any valid target for `Mapper.path`, which will be
        looked up on the current mapper instance and used to build the URL for
        that route. Additionally, it can be one of:

        '.'
            Builds the URL for the current route.

        '/'
            Builds the URL for the root (top-most) mapper instance.

        '/a', '/a.b', etc.
            Builds the URL for a named route relative to the root mapper.

        '.a', '..a', '..a.b', etc.
            Builds a URL for a named route relative to the current mapper.
            Each additional leading '.' after the first one starts one
            level higher in the hierarchy of nested mappers (i.e. '.a' is
            equivalent to 'a').

        """
        # Allow passing 'self' as named parameter
        self, target = args
        query = kw.pop('_query', None)
        relative = kw.pop('_relative', False)
        url = build_url(self._context, target, kw)
        if query:
            if isinstance(query, dict):
                query = sorted(query.items())
            url = url + '?' + urllib.urlencode(query)
        if relative:
            return url
        else:
            if self._application_uri is None:
                self._application_uri = application_uri(self.environ)
            return urlparse.urljoin(self._application_uri, url)

    @property
    def method(self):
        """The HTTP request method (verb)."""
        return self.environ.get('REQUEST_METHOD', 'GET').upper()

    @property
    def script_name(self):
        """The SCRIPT_NAME environment key as a unicode string."""
        return self.environ.get('SCRIPT_NAME', '').decode('utf-8')

    @property
    def path_info(self):
        """The PATH_INFO environment key as a unicode string."""
        return self.environ.get('PATH_INFO', '').decode('utf-8')

    @property
    def query(self):
        """A QueryDict instance holding the query parameters (QUERY_STRING)."""
        if self._query is None:
            query_string = self.environ.get('QUERY_STRING')
            self._query = QueryDict([
                (k.decode('utf-8'), v.decode('utf-8'))
                for k, v in urlparse.parse_qsl(query_string)
            ])
        return self._query

    @property
    def url(self):
        """The reconstructed request URI (absolute)."""
        if self._url is None:
            self._url = request_uri(self.environ, include_query=1)
        return self._url

    @property
    def content_type(self):
        """The value of the Content-Type header, or None"""
        return self.environ.get('CONTENT_TYPE')

    @property
    def content_length(self):
        """The value of the Content-Length header as an integer, or 0"""
        try:
            return int(self.environ['CONTENT_LENGTH'])
        except (KeyError, ValueError):
            return 0

    @property
    def server_name(self):
        """The SERVER_NAME environment key"""
        return self.environ.get('SERVER_NAME')

    @property
    def server_port(self):
        """The server's port number as an integer, or None (SERVER_PORT)."""
        port = self.environ.get('SERVER_PORT')
        return int(port) if port is not None else None

    @property
    def server_protocol(self):
        """The SERVER_PROTOCOL environment key"""
        return self.environ.get('SERVER_PROTOCOL')

    @property
    def remote_addr(self):
        """The REMOTE_ADDR environment key"""
        return self.environ.get('REMOTE_ADDR')

    @property
    def remote_port(self):
        """The remote hosts's port number as an integer, or None (REMOTE_PORT)."""
        port = self.environ.get('REMOTE_PORT')
        return int(port) if port is not None else None

    @property
    def scheme(self):
        """The URL scheme, usually 'http' or 'https' (wsgi.url_scheme)."""
        return self.environ.get('wsgi.url_scheme')

    @property
    def routing_args(self):
        """Returns named parameters extracted from the URL during routing."""
        self.environ.setdefault('wsgiorg.routing_args', ([], {}))
        return self.environ['wsgiorg.routing_args'][1]

    # TODO more CGI variables? See: <http://web.archive.org/web/20131002054457/http://ken.coar.org/cgi/draft-coar-cgi-v11-03.txt>

    # TODO file-like object that reads no more than content_length bytes
    # from wsgi.environ
    @property
    def body(self):
        """Reads content_length bytes from wsgi.input and returns the result.

        Cached after first access."""
        if self._body is None:
            self._body = self.environ['wsgi.input'].read(self.content_length)
        return self._body

    @property
    def form(self):
        """Reads the request body and tries to parse it as a web form.

        Parsing is done using the stdlib's cgi.FieldStorage which supports
        multipart forms (file uploads).
        Returns a QueryDict instance holding the form fields. Uploaded files
        are represented as form fields with a 'filename' attribute.
        """
        if self._form is None:
            environ = self.environ.copy()
            environ['QUERY_STRING'] = ''
            fs = cgi.FieldStorage(
                fp=self.environ['wsgi.input'],
                environ=environ,
                keep_blank_values=True)
            # File upload field handling copied from WebOb
            fields = []
            for f in fs.list or []:
                if f.filename:
                    f.filename = f.filename.decode('utf-8')
                    fields.append((f.name.decode('utf-8'), f))
                else:
                    fields.append(
                        (f.name.decode('utf-8'), f.value.decode('utf-8'))
                    )
            self._form = QueryDict(fields)
        return self._form

    @property
    def cookies(self):
        """Returns a dictionary mapping cookie names to their values."""
        if self._cookies is None:
            c = SimpleCookie(self.environ.get('HTTP_COOKIE'))
            self._cookies = dict([
                (k.decode('utf-8'), v.value.decode('utf-8'))
                for k, v in c.items()
            ])
        return self._cookies



from __future__ import absolute_import

import mimetypes
import os
from hashlib import md5

from .errors import NotFound, MethodNotAllowed
from .response import ok


class StaticFile(object):
    """Resource for serving a static file."""

    block_size = 65536
    default_content_type = 'application/octet-stream'

    def __init__(self, path, content_type=None, expires=None):
        if not os.path.isfile(path):
            raise ValueError("No such file: %s" % path)
        if content_type is None:
            content_type = mimetypes.guess_type(path)[0] \
                    or self.default_content_type
        self.path = path
        self.content_type = content_type
        self.expires = expires

    def __call__(self, request):
        if request.method not in ('GET', 'HEAD'):
            raise MethodNotAllowed(allow='GET, HEAD')
        stat = os.stat(self.path)
        etag = md5('%d:%f:%d' % (
            stat.st_ino, stat.st_mtime, stat.st_size)).hexdigest()

        def body():
            with open(self.path) as f:
                while True:
                    chunk = f.read(self.block_size)
                    if chunk == '':
                        break
                    yield chunk

        return ok(
                body, content_length=str(stat.st_size),
                content_type=self.content_type,
                etag=etag, expires=self.expires)


class StaticDirectory(object):
    """Resource for serving static files from a directory.

    Must be mapped to a route including a template parameter named
    `path`. Example::

        from rhino import Mapper, StaticDirectory

        app = Mapper()
        app.add('/static/{path:any}', StaticDirectory('./static'))

    Setting the `path` parameter's range to `any`, which includes '/', enables
    serving files from subdirectories of './static'.
    """

    # TODO add support for index.html, directory listings?
    def __init__(self, root, expires=None):
        self.root = os.path.abspath(root)
        self.expires = expires

    def __call__(self, request):
        # Normalize path_info to always start with a slash.
        path_info = '/' + request.routing_args.get('path', '').lstrip('/')
        # Interpret path_info as an OS path, resolve any non-leading '..', and
        # require resulting path to be absolute.
        # This is to prevent enumeration of directory names: If 'foo.txt' is
        # a public file, and a request for '../bar/foo.txt' succeeds, we know
        # that foo is in a directory named 'bar'.
        request_path = os.path.normpath(path_info)
        if not os.path.isabs(request_path):
            raise NotFound
        # Concatenate with root path and do prefix check
        prefix = os.path.abspath(self.root) + os.path.sep
        # Use '+' instead of os.path.join because request_path is absolute.
        filepath = os.path.abspath(prefix + request_path)
        if os.path.commonprefix([prefix, filepath]) != prefix:
            raise NotFound
        try:
            return StaticFile(filepath, expires=self.expires)(request)
        except ValueError:
            raise NotFound



from functools import partial

import mock
from mock import call

from rhino.errors import NotFound
from rhino.mapper import Mapper
from rhino.resource import Resource, get
from rhino.response import ok
from rhino.test import TestClient


class Wrapper(object):
    def __init__(self, wrapped):
        self.wrapped = wrapped
        self.request = None
        self.response = None
        self.cb = mock.create_autospec(lambda *args, **kw: 1)

    def __call__(self, request, ctx):
        self.request = request
        ctx.add_callback('enter', partial(self.cb, 'enter'))
        ctx.add_callback('leave', partial(self.cb, 'leave'))
        ctx.add_callback('finalize', partial(self.cb, 'finalize'))
        ctx.add_callback('teardown', partial(self.cb, 'teardown'))
        ctx.add_callback('close', partial(self.cb, 'close'))
        self.response = self.wrapped(request, ctx)
        return self.response


def test_callbacks():
    @get
    def handler(request):
        return ok('test')

    wrapper = Wrapper(Resource(handler))

    app = Mapper()
    app.add('/', wrapper)

    client = TestClient(app.wsgi)
    res = client.get('/')
    assert res.code == 200

    assert wrapper.cb.mock_calls == [
        call('enter', wrapper.request),
        call('leave', wrapper.request, wrapper.response),
        call('finalize', wrapper.request, wrapper.response),
        call('teardown'),
        call('close'),
    ]


def test_callbacks_exception():
    not_found = NotFound()

    @get
    def handler(request):
        raise not_found

    wrapper = Wrapper(Resource(handler))

    app = Mapper()
    app.add('/', wrapper)

    client = TestClient(app.wsgi)
    res = client.get('/')
    assert res.code == 404

    assert wrapper.cb.mock_calls == [
        call('enter', wrapper.request),
        call('finalize', wrapper.request, not_found.response),
        call('teardown'),
        call('close'),
    ]



# encoding: utf-8
from StringIO import StringIO
from wsgiref.util import setup_testing_defaults

import rhino
from mock import patch
from pytest import raises as assert_raises
from rhino.request import Request, QueryDict

body = 'x=1&x=2&%E2%98%85=%E2%98%83'
body_multipart = u'''--xxx
Content-Disposition: form-data; name="★"

☃
--xxx
Content-Disposition: form-data; name="★★"; filename="☃.txt"
Content-Type: text/plain

☃☃☃
--xxx--'''.encode('utf-8')

environ = {
    'REQUEST_METHOD': 'post',
    # SCRIPT_NAME and PATH_INFO are unquoted in the WSGI environment
    'SCRIPT_NAME': u'/☃'.encode('utf-8'),
    'PATH_INFO': u'/★'.encode('utf-8'),
    'QUERY_STRING': 'a=1&a=2&b=%E2%98%83',
    'HTTP_X_FOO': u'Smørebrød'.encode('latin-1'),
    'HTTP_COOKIE': 'x="\\342\\230\\203"; a=b',
    'CONTENT_TYPE': 'application/x-www-form-urlencoded',
    'CONTENT_LENGTH': str(len(body)),
    'REMOTE_ADDR': '1.2.3.4',
    'REMOTE_PORT': '12345',
    'wsgi.input': StringIO(body),
}
setup_testing_defaults(environ)

environ_multipart = {
    'REQUEST_METHOD': 'POST',
    'CONTENT_TYPE': 'multipart/form-data; boundary=xxx',
    'CONTENT_LENGTH': str(len(body_multipart)),
    'wsgi.input': StringIO(body_multipart),
}
setup_testing_defaults(environ_multipart)


def test_defaults():
    req = Request({})
    assert req.method == 'GET'
    assert 'Content-Length' not in req.headers
    assert req.content_length == 0


def test_empty_content_length():
    req = Request({'CONTENT_LENGTH': ''})
    assert req.content_length == 0


def test_bad_content_length():
    req = Request({'CONTENT_LENGTH': 'test'})
    assert req.content_length == 0


def test_accessors():
    req = Request(environ)
    assert req.method == 'POST'
    assert req.script_name == u'/☃'
    assert req.path_info == u'/★'
    # TODO Decode url?
    assert req.url == 'http://127.0.0.1/%E2%98%83/%E2%98%85?a=1&a=2&b=%E2%98%83'
    assert req.content_type == 'application/x-www-form-urlencoded'
    assert req.content_length == len(body)
    assert req.server_name == '127.0.0.1'
    assert req.server_port == 80
    assert req.server_protocol == 'HTTP/1.0'
    assert req.scheme == 'http'
    assert req.query.items() == [('a', '1'), ('a', '2'), ('b', u'☃')]
    assert req.form.items() == [('x', '1'), ('x', '2'), (u'★', u'☃')]
    assert req.cookies['x'] == u'☃'
    assert req.cookies['a'] == 'b'
    assert req.remote_addr == '1.2.3.4'
    assert req.remote_port == 12345
    assert req.body == ''


def test_request_headers():
    headers = Request(environ).headers
    content_length = str(len(body))
    assert headers['Host'] == '127.0.0.1'
    assert headers['hOST'] == '127.0.0.1'
    assert headers['Content-Type'] == 'application/x-www-form-urlencoded'
    assert headers['cONTENT-tYPE'] == 'application/x-www-form-urlencoded'
    assert headers['Content-Length'] == content_length
    assert headers['cONTENT-lENGTH'] == content_length
    assert headers['no-such-header'] is None
    assert headers['X-Foo'] == u'Smørebrød'
    assert set(headers.keys()) == set([
        'Host', 'Content-Type', 'Content-Length', 'X-Foo', 'Cookie',
    ])
    assert set(headers.values()) == set([
        '127.0.0.1', 'application/x-www-form-urlencoded', content_length,
        u'Smørebrød', 'x="\\342\\230\\203"; a=b',
    ])
    assert sorted(headers.items()) == [
        ('Content-Length', content_length),
        ('Content-Type', 'application/x-www-form-urlencoded'),
        ('Cookie', 'x="\\342\\230\\203"; a=b'),
        ('Host', '127.0.0.1'),
        ('X-Foo', u'Smørebrød'),
    ]
    assert len(headers) == 5


def test_mutable_routing_args():
    req = Request({})
    assert req.routing_args == {}
    req.routing_args['bar'] = 1
    assert req.routing_args == {'bar': 1}


def test_querydict():
    q = QueryDict([])
    assert 'z' not in q
    with assert_raises(KeyError):
        q['z']

    q = QueryDict([('a', 1), ('a', 2), ('b', 3)])
    assert q.keys() == ['a', 'a', 'b']
    assert q.values() == [1, 2, 3]
    assert q.items() == [('a', 1), ('a', 2), ('b', 3)]
    assert 'a' in q
    assert q['a'] == 1
    assert q.get('a') == 1
    assert q.get_all('a') == [1, 2]
    assert len(q) == 3

    q = QueryDict([('foo', None)])
    assert 'foo' in q
    assert q['foo'] is None


def test_file_upload():
    req = Request(environ_multipart)
    assert set(req.form.keys()) == set([u'★', u'★★'])
    assert req.form[u'★'] == u'☃'
    assert req.form[u'★★'].filename == u'☃.txt'
    assert req.form[u'★★'].type == 'text/plain'
    assert req.form[u'★★'].file.read() == u'☃☃☃'.encode('utf-8')


def test_url_for():
    req = Request(environ)
    with patch.object(rhino.request, 'build_url') as mock_url:
        mock_url.return_value = '/'
        assert req.url_for('/') == 'http://127.0.0.1/'
        assert req.url_for('/', _query={'foo': 'bar'}) == 'http://127.0.0.1/?foo=bar'
        assert req.url_for('/', _query=[('foo', 1), ('foo', 2)]) == 'http://127.0.0.1/?foo=1&foo=2'
        assert req.url_for('/', _relative=True) == '/'



from __future__ import absolute_import

import functools
import inspect


def _sse_encode(k, v):
    return ''.join('%s: %s\n' % (k, line) for line in v.split('\n'))


def sse_event(event=None, data=None, id=None, retry=None, comment=None):
    """Encode a Server-Sent Event (SSE).

    At least one field must be present. All fields are strings, except
    retry, which should be an integer. The data and comment fields can contain
    newlines.
    """
    if all(x is None for x in [event, data, id, retry, comment]):
        raise TypeError("Event must have at least one field")
    return ''.join([
        _sse_encode('', comment) if comment is not None else '',
        _sse_encode('event', event) if event is not None else '',
        _sse_encode('id', id) if id is not None else '',
        _sse_encode('retry', str(retry)) if retry is not None else '',
        _sse_encode('data', data) if data is not None else '',
        '\n',
    ])


def dual_use_decorator(fn):
    """Turn a function into a decorator that can be called with or without
    arguments."""
    @functools.wraps(fn)
    def decorator(*args, **kw):
        if len(args) == 1 and not kw and callable(args[0]):
            return fn()(args[0])
        else:
            return fn(*args, **kw)
    return decorator


def dual_use_decorator_method(fn):
    """Turn a method into a decorator that can be called with or without
    arguments. """
    @functools.wraps(fn)
    def decorator(*args, **kw):
        if len(args) == 2 and not kw and callable(args[1]):
            return fn(args[0])(args[1])
        else:
            return fn(*args, **kw)
    return decorator


def get_args(obj):
    """Get a list of argument names for a given callable."""
    if inspect.isfunction(obj):
        return inspect.getargspec(obj).args
    elif inspect.ismethod(obj):
        return inspect.getargspec(obj).args[1:]
    elif inspect.isclass(obj):
        return inspect.getargspec(obj.__init__).args[1:]
    elif hasattr(obj, '__call__'):
        return inspect.getargspec(obj.__call__).args[1:]
    else:
        raise TypeError("Can't inspect signature of '%s' object." % obj)


def call_with_ctx(fn, ctx, *args, **kw):
    """Call fn with or without 'ctx', depending on its signature."""
    if 'ctx' in get_args(fn):
        return fn(*args, ctx=ctx, **kw)
    else:
        return fn(*args, **kw)



import json
from cgi import escape

from rhino import Mapper, Resource, ok
from rhino.errors import BadRequest

greeting = Resource()
greeting.text = "Hello, world!"

@greeting.get(provides='text/html')
def get_html(request, ctx):
    return '<html><h1>%s</h1></html>' % escape(greeting.text)

@greeting.get(provides='application/json')
def get_json(request, ctx):
    return json.dumps({'greeting': greeting.text})

@greeting.put(accepts='application/json')
def put_json(request, ctx):
    try:
        data = json.loads(request.body)
        greeting.text = data['greeting']
    except Exception:
        raise BadRequest
    return ok()

app = Mapper()
app.add('/', greeting)

if __name__ == '__main__':
    app.start_server()



#!/usr/bin/env python
from setuptools import setup, find_packages

version = '0.0.1'

with open('README.rst') as f:
    README = f.read()

setup(
    name='Rhino',
    version=version,
    author='Stanis Trendelenburg',
    author_email='stanis.trendelenburg@gmail.com',
    packages=find_packages(exclude=['test*', 'example*']),
    url='https://github.com/trendels/rhino',
    license='MIT',
    description='A microframework for building RESTful web services',
    long_description=README,
    classifiers = [
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
    ],
)



from __future__ import absolute_import

import types
from collections import defaultdict, namedtuple

from .errors import NotFound, MethodNotAllowed, UnsupportedMediaType, \
        NotAcceptable
from .response import Response
from .util import dual_use_decorator, dual_use_decorator_method, call_with_ctx
from .vendor import mimeparse

class_types = (type, types.ClassType)  # new-style and old-style classes

VIEW_SEPARATOR = ':'
MIMEPARSE_NO_MATCH = (-1, 0)

class handler_metadata(namedtuple(
        'handler_metadata', 'verb view accepts provides')):
    @classmethod
    def create(cls, verb, view=None, accepts='*/*', provides=None):
        if view and VIEW_SEPARATOR in view:
            raise ValueError("View name cannot contain '%s': %s"
                    % (VIEW_SEPARATOR, view))
        return cls(verb, view, accepts, provides)


def _make_handler_decorator(*args, **kw):
    def decorator(fn):
        fn._rhino_meta = handler_metadata.create(*args, **kw)
        return fn
    return decorator


@dual_use_decorator
def get(*args, **kw):
    """Mark the decorated function as a handler for GET requests."""
    return _make_handler_decorator('GET', *args, **kw)


@dual_use_decorator
def post(*args, **kw):
    """Mark the decorated function as a handler for POST requests."""
    return _make_handler_decorator('POST', *args, **kw)


@dual_use_decorator
def put(*args, **kw):
    """Mark the decorated function as a handler for PUT requests."""
    return _make_handler_decorator('PUT', *args, **kw)


@dual_use_decorator
def delete(*args, **kw):
    """Mark the decorated function as a handler for DELETE requests."""
    return _make_handler_decorator('DELETE', *args, **kw)


@dual_use_decorator
def patch(*args, **kw):
    """Mark the decorated function as a handler for PATCH requests."""
    return _make_handler_decorator('PATCH', *args, **kw)


@dual_use_decorator
def options(*args, **kw):
    """Mark the decorated function as a handler for OPTIONS requests."""
    return _make_handler_decorator('OPTIONS', *args, **kw)


def make_response(obj):
    """Try to coerce an object into a Response instance."""
    if obj is None:
        raise TypeError("Handler return value cannot be None.")
    if isinstance(obj, Response):
        return obj
    return Response(200, body=obj)


def resolve_handler(request, view_handlers):
    """Select a suitable handler to handle the request.

    Returns a (handler, vary) tuple, where handler is a handler_metadata tuple
    and vary is a set containing header names that were used during content
    negotiation and that should to be included in the 'Vary' header of the
    outgoing response.

    When no suitable handler exists, raises NotFound, MethodNotAllowed,
    UnsupportedMediaType or NotAcceptable.
    """
    view = None
    if request._context:  # Allow context to be missing for easier testing
        route_name = request._context[-1].route.name
        if route_name and VIEW_SEPARATOR in route_name:
            view = route_name.split(VIEW_SEPARATOR, 1)[1] or None

    if view not in view_handlers:
        raise NotFound

    method_handlers = view_handlers[view]

    verb = request.method.upper()
    if verb not in method_handlers:
        if verb == 'HEAD' and 'GET' in method_handlers:
            verb = 'GET'
        else:
            allowed_methods = set(method_handlers.keys())
            if 'HEAD' not in allowed_methods and 'GET' in allowed_methods:
                allowed_methods.add('HEAD')
            allow = ', '.join(sorted(allowed_methods))
            raise MethodNotAllowed(allow=allow)

    handlers = method_handlers[verb]
    vary = set()
    if len(set(h.provides for h in handlers if h.provides is not None)) > 1:
        vary.add('Accept')
    if len(set(h.accepts for h in handlers if h.accepts != '*/*')) > 1:
        vary.add('Content-Type')

    content_type = request.content_type
    if content_type:
        handlers = negotiate_content_type(content_type, handlers)
        if not handlers:
            raise UnsupportedMediaType

    accept = request.headers.get('Accept')
    if accept:
        handlers = negotiate_accept(accept, handlers)
        if not handlers:
            raise NotAcceptable

    handler = handlers[0]
    return handler, vary


def negotiate_content_type(content_type, handlers):
    """Filter handlers that accept a given content-type.

    Finds the most specific media-range that matches `content_type`, and
    returns those handlers that accept it.
    """
    accepted = [h.accepts for h in handlers]
    scored_ranges = [(mimeparse.fitness_and_quality_parsed(content_type,
        [mimeparse.parse_media_range(mr)]), mr) for mr in accepted]

    # Sort by fitness, then quality parsed (higher is better)
    scored_ranges.sort(reverse=True)
    best_score = scored_ranges[0][0]  # (fitness, quality)
    if best_score == MIMEPARSE_NO_MATCH or not best_score[1]:
        return []

    media_ranges = [pair[1] for pair in scored_ranges if pair[0] == best_score]
    best_range = media_ranges[0]
    return [h for h in handlers if h.accepts == best_range]


def negotiate_accept(accept, handlers):
    """Filter handlers that provide an acceptable mime-type.

    Finds the best match among handlers given an Accept header, and returns
    those handlers that provide the matching mime-type.
    """
    provided = [h.provides for h in handlers]
    if None not in provided:
        # All handlers are annotated with the mime-type they
        # provide: find the best match.
        #
        # mimeparse.best_match expects the supported mime-types to be sorted
        # in order of increasing desirability. By default, we use the order in
        # which handlers were added (earlier means better).
        # TODO: add "priority" parameter for user-defined priorities.
        best_match = mimeparse.best_match(reversed(provided), accept)
        return [h for h in handlers if h.provides == best_match]
    else:
        # Not all handlers are annotated - disable content-negotiation
        # for Accept.
        # TODO: Enable "Optimistic mode": If a fully qualified mime-type was
        # requested and we have a specific handler it, return that instead of
        # the default handler (depending on 'q' value).
        return [h for h in handlers if h.provides is None]


class Resource(object):
    """
    Represents a REST resource.

    This class can be used in multiple ways:

    As a standalone resource, using it's methods to register handlers::

        my_resource = Resource()

        @my_resource.get
        def get_resource(request):
            pass

    As a class decorator for class-based resources::

        @Resource
        class MyResource(object):
            @get
            def index(self, request):
                pass

    As a wrapper to create resouces from custom objects::

        class MyClass(object):
            def __init_(self, *args):
                pass

            @get
            def index(self, request):
                pass

        my_resource = Resource(MyClass())

    When used as a wrapper or class decorator, handlers will be picked up from
    methods of the wrapped instance or class that have been decorated with one
    of the decorator functions provided by this module (`get`, `post`, etc.)

    Additionally, if the wrapped object implements `from_url`, that method will
    be called before any handler to filter the URL parameters that will be
    passed to the handler as keyword arguments.

    When used as a standalone instance, functions can be registered as handlers
    using the instance's methods, as shown above. The `from_url` method can be
    used in the same way to register a filter for URL parameters.
    """

    def __init__(self, wrapped=None):
        self._wrapped = wrapped
        self._handlers = defaultdict(lambda: defaultdict(list))
        self._handler_lookup = {}
        self._from_url = None
        if wrapped is not None:
            if hasattr(wrapped, '_rhino_meta'):
                meta = wrapped._rhino_meta
                self._handlers[meta.view][meta.verb].append(meta)
                self._handler_lookup[meta] = wrapped
            else:
                for name in dir(wrapped):
                    prop = getattr(wrapped, name)
                    if hasattr(prop, '_rhino_meta'):
                        meta = prop._rhino_meta
                        self._handlers[meta.view][meta.verb].append(meta)
                        self._handler_lookup[meta] = prop

    def __call__(self, request, ctx):
        resource_is_class = type(self._wrapped) in class_types
        resource = self._wrapped() if resource_is_class else self._wrapped
        try:
            handler, vary = resolve_handler(request, self._handlers)
        except MethodNotAllowed as e:
            # Handle 'OPTIONS' requests by default
            allow = e.response.headers.get('Allow', '')
            allowed_methods = set([s.strip() for s in allow.split(',')])
            allowed_methods.add('OPTIONS')
            allow = ', '.join(sorted(allowed_methods))
            if request.method.upper() == 'OPTIONS':
                return Response(200, headers=[('Allow', allow)])
            else:
                e.response.headers['Allow'] = allow
                raise

        ctx._run_callbacks('enter', request)

        url_args_filter = self._from_url or getattr(resource, 'from_url', None)
        kw = request.routing_args
        if url_args_filter:
            kw = call_with_ctx(url_args_filter, ctx, request, **kw)

        fn = self._handler_lookup[handler]
        if resource_is_class:
            response = make_response(call_with_ctx(fn, ctx, resource, request, **kw))
        else:
            response = make_response(call_with_ctx(fn, ctx, request, **kw))

        ctx._run_callbacks('leave', request, response)

        if handler.provides:
            response.headers.setdefault('Content-Type', handler.provides)

        if vary:
            vary_header = response.headers.get('Vary', '')
            vary_items = set(filter(
                None, [s.strip() for s in vary_header.split(',')]))
            vary_items.update(vary)
            response.headers['Vary'] = ', '.join(sorted(vary_items))
        return response

    def _make_decorator(self, *args, **kw):
        def decorator(fn):
            meta = handler_metadata.create(*args, **kw)
            self._handlers[meta.view][meta.verb].append(meta)
            self._handler_lookup[meta] = fn
            return fn
        return decorator

    @dual_use_decorator_method
    def get(self, *args, **kw):
        """Install the decorated function as a handler for GET requests."""
        return self._make_decorator('GET', *args, **kw)

    @dual_use_decorator_method
    def post(self, *args, **kw):
        """Install the decorated function as a handler for POST requests."""
        return self._make_decorator('POST', *args, **kw)

    @dual_use_decorator_method
    def put(self, *args, **kw):
        """Install the decorated function as a handler for PUT requests."""
        return self._make_decorator('PUT', *args, **kw)

    @dual_use_decorator_method
    def delete(self, *args, **kw):
        """Install the decorated function as a handler for DELETE requests."""
        return self._make_decorator('DELETE', *args, **kw)

    @dual_use_decorator_method
    def patch(self, *args, **kw):
        """Install the decorated function as a handler for PATCH requests."""
        return self._make_decorator('PATCH', *args, **kw)

    @dual_use_decorator_method
    def options(self, *args, **kw):
        """Install the decorated function as a handler for OPTIONS requests."""
        return self._make_decorator('OPTIONS', *args, **kw)

    def from_url(self, fn):
        """Install the decorated function as a filter for URL parameters."""
        self._from_url = fn
        return fn



from __future__ import absolute_import

from collections import namedtuple

request_context = namedtuple('request_context', 'root mapper route')
string_types = (str, unicode)


def build_url(context, target, params=None):
    if not context:  # pragma: no cover
        raise RuntimeError("No routing context present.")
    if params is None:
        params = {}
    if type(target) in string_types and len(target) \
            and (target[0] == '/' or '.' in target):
        # Build URL for a relative or absolute route name
        if target == '.':  # The current route
            c = context[-1]
            return c.root + c.mapper.path(c.route, params)
        elif target == '/':  # The root mapper instance
            return context[0].root or '/'
        elif target[0] == '/':  # A dotted route name anchored at the root level
            c = context[0]
            return c.root + c.mapper.path(target[1:], params)
        else:  # A route name relative to the current mapper
            rel_name = target.lstrip('.')
            leading_dots = target[:-len(rel_name)]
            c = context[-len(leading_dots)]
            return c.root + c.mapper.path(rel_name, params)
    else:
        # Try resolving the target via the current mapper
        c = context[-1]
        return c.root + c.mapper.path(target, params)



# encoding: utf-8
import unittest

from rhino.mapper import Mapper, template2regex, template2path, \
        InvalidArgumentError, InvalidTemplateError
from rhino.response import Response

# Dispatcher and template2regex tests taken from Joe Gregorio's
# wsgidispatcher.py (https://code.google.com/p/robaccia/) with minor
# modifications.

Dispatcher = Mapper

class Test(unittest.TestCase):
    def setUp(self):
        self.called = False
        self.status = None
        self.app_number = 0

    def _start_response(self, status, response_headers, exc_info=None):
        self.status = status

    def _app(self, request):
        self.called = True
        self.environ = request.environ.copy()
        return Response(200, [], '')

    def _app1(self, request):
        self.app_number = 1
        return self._app(request)

    def _app2(self, request):
        self.app_number = 2
        return self._app(request)

    def _app3(self, request):
        self.app_number = 3
        return self._app(request)

    def test_happy_path(self):
        d = Dispatcher()
        d.add("/fred/", self._app)
        d.wsgi({'PATH_INFO': '/fred/', 'REQUEST_METHOD': 'GET'}, self._start_response)
        self.assertTrue(self.called)

    def test_happy_path_miss(self):
        d = Dispatcher()
        d.add("/fred/", self._app)
        d.wsgi({'PATH_INFO': '/fred', 'REQUEST_METHOD': 'GET'}, self._start_response)
        self.assertFalse(self.called)

    def test_zero_length_path(self):
        d = Dispatcher()
        d.add("", self._app)
        d.wsgi({'PATH_INFO': '', 'REQUEST_METHOD': 'GET'}, self._start_response)
        self.assertTrue(self.called)

    def test_template_simple(self):
        d = Dispatcher()
        d.add("/{fred}/", self._app)
        d.wsgi({'PATH_INFO': '/barney/', 'REQUEST_METHOD': 'GET'}, self._start_response)
        self.assertTrue(self.called)
        self.assertEqual(self.status, "200 OK")
        self.assertEqual(self.environ['wsgiorg.routing_args'][1]['fred'], 'barney')

    def test_template_two_part(self):
        d = Dispatcher()
        d.add("/{name}/[{name2}/]", self._app)
        d.wsgi({'PATH_INFO': '//', 'REQUEST_METHOD': 'GET'}, self._start_response)
        self.assertFalse(self.called)
        d.wsgi({'PATH_INFO': '/fred', 'REQUEST_METHOD': 'GET'}, self._start_response)
        self.assertFalse(self.called)
        d.wsgi({'PATH_INFO': '/fred/barney', 'REQUEST_METHOD': 'GET'}, self._start_response)
        self.assertFalse(self.called)
        d.wsgi({'PATH_INFO': '/fred/barney/', 'REQUEST_METHOD': 'GET'}, self._start_response)
        self.assertTrue(self.called)
        self.assertEqual(self.environ['wsgiorg.routing_args'][1]['name'], 'fred')
        self.assertEqual(self.environ['wsgiorg.routing_args'][1]['name2'], 'barney')

    def test_template_two_part_trailing(self):
        d = Dispatcher()
        d.add("/{name}/[{name2}/]|", self._app)
        d.wsgi({'PATH_INFO': '//', 'REQUEST_METHOD': 'GET'}, self._start_response)
        self.assertFalse(self.called)
        d.wsgi({'PATH_INFO': '/fred', 'REQUEST_METHOD': 'GET'}, self._start_response)
        self.assertFalse(self.called)
        d.wsgi({'PATH_INFO': '/fred/barney', 'REQUEST_METHOD': 'GET'}, self._start_response)
        self.assertTrue(self.called)
        self.assertEqual(self.environ['wsgiorg.routing_args'][1]['name'], 'fred')
        #self.assertEqual(self.environ['wsgiorg.routing_args'][1]['name2'], None)
        self.assertTrue('name2' not in self.environ['wsgiorg.routing_args'][1])

    def test_template_new_char_range(self):
        d = Dispatcher({'real': '(\+|-)?[1-9]\.[0-9]*E(\+|-)?[0-9]+'})
        d.add("/{name:real}/", self._app)
        d.wsgi({'PATH_INFO': '/3.1415925535E-10/', 'REQUEST_METHOD': 'GET'}, self._start_response)
        self.assertEqual(self.environ['wsgiorg.routing_args'][1]['name'], '3.1415925535E-10')

    def test_real_world_examples(self):
        urls = Dispatcher()
        urls.add('/service/[{ctype:alpha}[/[{id:unreserved}/]]][;{noun}]', self._app1)
        urls.add('/comments/[{id:alnum}]',  self._app2)
        urls.add('/{alpha}/[{id}[/[{slug}]]]',  self._app3)

        urls.wsgi({'PATH_INFO': '/service/;service_document', 'REQUEST_METHOD': 'GET'}, self._start_response)
        self.assertEqual(self.status, "200 OK")
        self.assertEqual(1, self.app_number)
        self.assertEqual(self.environ['wsgiorg.routing_args'][1]['noun'], 'service_document')
        self.app_number = 0

        urls.wsgi({'PATH_INFO': '/comments/2', 'REQUEST_METHOD': 'GET'}, self._start_response)
        self.assertEqual(self.status, "200 OK")
        self.assertEqual(2, self.app_number)
        self.assertEqual(self.environ['wsgiorg.routing_args'][1]['id'], '2')
        self.app_number = 0

        urls.wsgi({'PATH_INFO': '/draft/98/My-slug_name', 'REQUEST_METHOD': 'GET'}, self._start_response)
        self.assertEqual(self.status, "200 OK")
        self.assertEqual(3, self.app_number)
        self.assertEqual(self.environ['wsgiorg.routing_args'][1]['slug'], 'My-slug_name')
        self.assertEqual(self.environ['wsgiorg.routing_args'][1]['id'], '98')

    def test_extreme_laziness(self):
        """No rules, no method, and no path_info. We should
        not throw an exception"""
        d = Dispatcher()
        d.wsgi({}, self._start_response)
        self.assertFalse(self.called)
        self.assertEqual(self.status, "404 Not Found")

    def test_semicolon(self):
        urls = Dispatcher()
        urls.add('/service/[{id:word}][;{noun}]', self._app)

        urls.wsgi({'PATH_INFO': '/service/fred;service_document', 'REQUEST_METHOD': 'GET'}, self._start_response)
        self.assertEqual(self.status, "200 OK")
        self.assertEqual(self.environ['wsgiorg.routing_args'][1]['noun'], 'service_document')
        self.assertEqual(self.environ['wsgiorg.routing_args'][1]['id'], 'fred')

    def test_unicode(self):
        urls = Dispatcher()
        urls.add('/{name}', self._app)

        urls.wsgi({'PATH_INFO': u'/☃'.encode('utf-8')}, self._start_response)
        self.assertEqual(self.status, "200 OK")
        self.assertEqual(self.environ['wsgiorg.routing_args'][1]['name'], u'☃')


class Template2Regex(unittest.TestCase):

    def test_template_failures(self):
        cases = ('[][', '[]]', '{}{', '{}}', '|a', 'a|b')
        for template in cases:
            self.assertRaises(InvalidTemplateError, template2regex, template)

    def test_template_expand(self):
        cases = [
                ("{fred}", (r"^(?P<fred>[^/]+)$", ['fred'])),
                ("{fred:alpha}", (r"^(?P<fred>[a-zA-Z]+)$", ['fred'])),
                ("{fred:unreserved}", (r"^(?P<fred>[a-zA-Z\d\-\.\_\~]+)$", ['fred'])),
                ("{fred}|", (r"^(?P<fred>[^/]+)", ['fred'])),
                ("{fred}/{barney}|", (r"^(?P<fred>[^/]+)\/(?P<barney>[^/]+)", ['fred', 'barney'])),
                ("{fred}[/{barney}]|", (r"^(?P<fred>[^/]+)(\/(?P<barney>[^/]+))?", ['fred', 'barney'])),
                ("{fred}[/[{barney}]]|", (r"^(?P<fred>[^/]+)(\/((?P<barney>[^/]+))?)?", ['fred', 'barney'])),
                ("{fred}[/[{barney}]]", (r"^(?P<fred>[^/]+)(\/((?P<barney>[^/]+))?)?$", ['fred', 'barney'])),
                ("/{id}[/[{slug}]];edit_comment_form", (r"^\/(?P<id>[^/]+)(\/((?P<slug>[^/]+))?)?\;edit\_comment\_form$", ['id', 'slug'])),
                ("/service/[{ctype:alpha}[/[{id}/]]][;{noun}]", (r"^\/service\/((?P<ctype>[a-zA-Z]+)(\/((?P<id>[^/]+)\/)?)?)?(\;(?P<noun>[^/]+))?$", ['ctype', 'id', 'noun'])),
                ]
        for template, result in cases:
            self.assertEqual(template2regex(template), result)


class TestTemplate2Path(unittest.TestCase):

    def test_example(self):
        template = "/service/[{collection:alpha}[/[{id:unreserved}/]]][;{noun}]"
        self.assertEqual(template2path(template, {}), '/service/')
        self.assertEqual(template2path(template, {'collection':'posts'}), '/service/posts')
        self.assertEqual(template2path(template, {'collection':'posts', 'id':123}), '/service/posts/123/')
        self.assertEqual(template2path(template, {'collection':'posts', 'id':123, 'noun':'form'}), '/service/posts/123/;form')

    def test_params(self):
        template = "/{a}"
        self.assertEqual(template2path(template, {'a':'x'}), '/x')
        self.assertEqual(template2path(template, {'a':'x', 'b':'y'}), '/x')
        self.assertRaises(InvalidArgumentError, template2path, template, {})

    def test_ranges(self):
        template = "/[{a:myrange}]"
        self.assertEqual(template2path(template, {}), '/')
        self.assertEqual(template2path(template, {'a':1}, {'myrange':'\d+'}), '/1')
        self.assertRaises(InvalidArgumentError, template2path, template, {'a':'x'}, {'myrange':'\d+'})

    def test_brackets(self):
        template = "/[{a}]-[x]-[{b}]"
        self.assertEqual(template2path(template, {'a':1, 'b':2}), '/1--2')
        self.assertEqual(template2path(template, {'a':1       }), '/1--')
        self.assertEqual(template2path(template, {       'b':2}), '/--2')
        self.assertEqual(template2path(template, {            }), '/--')

    def test_nested_lr(self):
        template = "/[{a}[-{b}[-{c}]]/]x"
        self.assertEqual(template2path(template, {'a':1, 'b':2, 'c':3}), '/1-2-3/x')
        self.assertEqual(template2path(template, {'a':1, 'b':2       }), '/1-2/x')
        self.assertEqual(template2path(template, {'a':1              }), '/1/x')
        # unused optional parameters don't raise an exception
        self.assertEqual(template2path(template, {'a':1,        'c':3}), '/1/x')
        self.assertEqual(template2path(template, {       'b':2, 'c':3}), '/x')
        self.assertEqual(template2path(template, {              'c':3}), '/x')
        self.assertEqual(template2path(template, {       'b':2       }), '/x')

    def test_nested_rl(self):
        template = "/[[[{a}-]{b}-]{c}/]x"
        self.assertEqual(template2path(template, {'a':1, 'b':2, 'c':3}), '/1-2-3/x')
        self.assertEqual(template2path(template, {       'b':2, 'c':3}), '/2-3/x')
        self.assertEqual(template2path(template, {              'c':3}), '/3/x')
        # unused optional parameters don't raise an exception
        self.assertEqual(template2path(template, {'a':1,        'c':3}), '/3/x')
        self.assertEqual(template2path(template, {'a':1, 'b':2       }), '/x')
        self.assertEqual(template2path(template, {'a':1,             }), '/x')
        self.assertEqual(template2path(template, {       'b':2       }), '/x')

    def test_bar(self):
        self.assertEqual(template2path('/a|', {}), '/a')

    def test_quote_reserved(self):
        self.assertEqual(template2path('/{x:any}', {'x': 'foo:;/bar?a=1&b=2'}), '/foo:;/bar%3Fa%3D1%26b%3D2')

    def test_unicode(self):
        self.assertEqual(template2path('/{name}', {'name': u'☃'}), '/%E2%98%83')

    def test_template_failures(self):
        cases = ('[][', '[]]', '{x}{', '{x}}', '|a', 'a|b')
        for template in cases:
            self.assertRaises(InvalidTemplateError,
                    template2path, template, {'x': 1})


def test_path():
    app = Mapper()
    fn = lambda: None
    app.add('/', fn, 'test')
    assert app.path('test', {}) == '/'
    assert app.path(fn, {}) == '/'



from __future__ import absolute_import

import httplib
import re
import time
from calendar import timegm
from collections import namedtuple
from datetime import datetime, timedelta
from email.utils import parsedate
from wsgiref.handlers import format_date_time

# The "unrolled" version is faster by about a factor of 2.
#_quoted_string = r'"(?:[^"\\]|\\.)*"'
_quoted_string = r'"[^"\\]*(?:\\.[^\\"]*)*"'
_etag_weak_prefix = r'[Ww]/'
_etag_star = r'\*'
_etag = r'(%s)?(%s)' % (_etag_weak_prefix, _quoted_string)
_etag_header = r'^(?:(%s)|(?:%s(?:\s*,\s*|$))+)$' % (_etag_star, _etag)

etag_re = re.compile(_etag)
etag_header_re = re.compile(_etag_header)
quoted_string_re = re.compile(_quoted_string)

status_codes = httplib.responses.copy()


def parse_etag_header(header):
    """Parse a header containing one or more ETags or a wildcard ('*').

    Returns the string '*' or a list of etags as (weak, etag) tuples.
    `weak` is the prefix designating a weak etag, or the empty string.
    `etag` is the etag (including quotes) with the weak prefix stripped
    off. Returns an empty list if the header could not be parsed.

    Example:

    >>> parse_etag_header('*')
    '*'
    >>> parse_etag_header('"foo"  ')
    [('', '"foo"')]
    >>> parse_etag_header('"foo", w/"bar", W/"baz"')
    [('', '"foo"'), ('w/', '"bar"'), ('W/', '"baz"')]
    >>> parse_etag_header('invalid')
    []

    """
    m = etag_header_re.match(header.strip())
    if not m:
        return []
    if m.group(1):  # star
        return m.group(1)
    else:  # list of entity tags
        return etag_re.findall(header)


def match_etag(etag, header, weak=False):
    """Try to match an ETag against a header value.

    If `weak` is True, uses the weak comparison function.
    """
    if etag is None:
        return False
    m = etag_re.match(etag)
    if not m:
        raise ValueError("Not a well-formed etag: '%s'" % etag)
    (is_weak, etag) = m.groups()
    parsed_header = parse_etag_header(header)
    if parsed_header == '*':
        return True
    if is_weak and not weak:
        return False
    if weak:
        return etag in [t[1] for t in parsed_header]
    else:
        return etag in [t[1] for t in parsed_header if not t[0]]


def datetime_to_timestamp(dt):
    """Convert datetime.datetime to Unix timestamp."""
    return timegm(dt.utctimetuple())


def httpdate_to_timestamp(s):
    """Convert HTTP date to Unix timestamp."""
    return timegm(parsedate(s))


def total_seconds(td):
    if hasattr(td, 'total_seconds'):  # Since Python 2.7
        return td.total_seconds()
    else:  # pragma: no cover
        return td.seconds + 60*60*24 * td.days + td.microseconds/1000000.0


def datetime_to_httpdate(dt):
    """Convert datetime.datetime or Unix timestamp to HTTP date."""
    if isinstance(dt, (int, float)):
        return format_date_time(dt)
    elif isinstance(dt, datetime):
        return format_date_time(datetime_to_timestamp(dt))
    else:
        raise TypeError("expected datetime.datetime or timestamp (int/float),"
                        " got '%s'" % dt)


def timedelta_to_httpdate(td):
    """Convert datetime.timedelta or number of seconds to HTTP date.

    Returns an HTTP date in the future.
    """
    if isinstance(td, (int, float)):
        return format_date_time(time.time() + td)
    elif isinstance(td, timedelta):
        return format_date_time(time.time() + total_seconds(td))
    else:
        raise TypeError("expected datetime.timedelta or number of seconds"
                        "(int/float), got '%s'" % td)


def cache_control(max_age=None, private=False, public=False, s_maxage=None,
        must_revalidate=False, proxy_revalidate=False, no_cache=False,
        no_store=False):
    """Generate the value for a Cache-Control header.

    Example:

    >>> from rhino.http import cache_control as cc
    >>> from datetime import timedelta
    >>> cc(public=1, max_age=3600)
    'public, max-age=3600'
    >>> cc(public=1, max_age=timedelta(hours=1))
    'public, max-age=3600'
    >>> cc(private=True, no_cache=True, no_store=True)
    'private, no-cache, no-store'

    """
    if all([private, public]):
        raise ValueError("'private' and 'public' are mutually exclusive")
    if isinstance(max_age, timedelta):
        max_age = int(total_seconds(max_age))
    if isinstance(s_maxage, timedelta):
        s_maxage = int(total_seconds(s_maxage))
    directives = []
    if public: directives.append('public')
    if private: directives.append('private')
    if max_age is not None: directives.append('max-age=%d' % max_age)
    if s_maxage is not None: directives.append('s-maxage=%d' % s_maxage)
    if no_cache: directives.append('no-cache')
    if no_store: directives.append('no-store')
    if must_revalidate: directives.append('must-revalidate')
    if proxy_revalidate: directives.append('proxy-revalidate')
    return ', '.join(directives)



from pytest import raises as assert_raises
from collections import defaultdict
from rhino.errors import NotFound, MethodNotAllowed, UnsupportedMediaType, \
        NotAcceptable
from rhino.mapper import Route, Context
from rhino.request import Request
from rhino.resource import Resource, negotiate_content_type, negotiate_accept, \
        resolve_handler, make_response, handler_metadata
from rhino.response import Response, ok
from rhino.test import make_handler_metadata


def test_negotiate_content_type():
    handlers = [
        make_handler_metadata(accepts='*/*'),
        make_handler_metadata(accepts='text/*'),
        make_handler_metadata(accepts='text/plain'),
        make_handler_metadata(accepts='text/plain'),
        make_handler_metadata(accepts='application/json'),
        make_handler_metadata(accepts='image/png;q=0.8'),
        make_handler_metadata(accepts='image/png;q=0.9'),
    ]
    assert negotiate_content_type('application/xml', handlers) == [handlers[0]]
    assert negotiate_content_type('text/html', handlers) == [handlers[1]]
    assert negotiate_content_type('text/plain', handlers) == handlers[2:4]
    assert negotiate_content_type('application/json', handlers) == [handlers[4]]
    assert negotiate_content_type('image/png', handlers) == [handlers[6]]


def test_negotiate_accept():
    handlers = [
        make_handler_metadata(provides='text/plain'),
        make_handler_metadata(provides='text/html'),
        make_handler_metadata(provides='application/json'),
    ]
    assert negotiate_accept('text/*', handlers) == [handlers[0]]
    assert negotiate_accept('text/plain;q=0.1, text/html', handlers) == [handlers[1]]
    assert negotiate_accept('text/*, application/json', handlers) == [handlers[2]]

    handlers = [
        make_handler_metadata(provides='text/plain'),
        make_handler_metadata(provides=None),
    ]
    assert negotiate_accept('text/*', handlers) == [handlers[1]]
    assert negotiate_accept('text/plain', handlers) == [handlers[1]]


def make_handler_dict():
    return defaultdict(lambda: defaultdict(list))


def add_handler(handler_dict, fn, verb, view=None, accepts='*/*', provides=None):
    handler = handler_metadata(fn, verb, view, accepts, provides)
    handler_dict[view][verb].append(handler)


def test_resolve_handler():
    assert_raises(NotFound, resolve_handler, Request({}), {})

    handlers = {
        None: {
            'GET': [
                make_handler_metadata(verb='GET', provides='text/html')
            ],
            'POST': [
                make_handler_metadata(verb='POST', accepts='text/plain', provides='text/plain'),
                make_handler_metadata(verb='POST', accepts='text/plain', provides='image/png'),
            ],
            'PUT': [
                make_handler_metadata(verb='PUT', accepts='text/plain'),
                make_handler_metadata(verb='PUT', accepts='application/json'),
            ],
        },
        'test': {
            'POST': [
                make_handler_metadata(verb='POST', accepts='text/plain', provides='text/plain'),
                make_handler_metadata(verb='POST', accepts='application/xml', provides='image/png'),
            ],
        }
    }
    get_handlers = handlers[None]['GET']
    post_handlers = handlers[None]['POST']
    put_handlers = handlers[None]['PUT']
    post_handlers_test_view = handlers['test']['POST']
    no_vary = set()
    vary_accept = set(['Accept'])
    vary_content_type = set(['Content-Type'])
    vary_both = vary_accept | vary_content_type

    rv = resolve_handler(Request({'REQUEST_METHOD': 'GET'}), handlers)
    assert rv == (get_handlers[0], no_vary)

    rv = resolve_handler(Request({'REQUEST_METHOD': 'HEAD'}), handlers)
    assert rv == (get_handlers[0], no_vary)

    assert_raises(MethodNotAllowed, resolve_handler, Request(
        {'REQUEST_METHOD': 'DELETE'}), handlers)

    rv = resolve_handler(Request(
        {'REQUEST_METHOD': 'GET', 'HTTP_ACCEPT': 'text/html'}), handlers)
    assert rv == (get_handlers[0], no_vary)

    assert_raises(NotAcceptable, resolve_handler, Request(
        {'REQUEST_METHOD': 'GET', 'HTTP_ACCEPT': 'text/plain'}), handlers)

    assert_raises(NotAcceptable, resolve_handler, Request(
        {'REQUEST_METHOD': 'GET', 'HTTP_ACCEPT': 'text/plain'}), handlers)

    assert_raises(UnsupportedMediaType, resolve_handler, Request(
        {'REQUEST_METHOD': 'POST', 'CONTENT_TYPE': 'text/html'}), handlers)

    rv = resolve_handler(Request(
        {'REQUEST_METHOD': 'POST',
         'CONTENT_TYPE': 'text/plain',
         'HTTP_ACCEPT': 'text/plain'}), handlers)
    assert rv == (post_handlers[0], vary_accept)

    rv = resolve_handler(Request(
        {'REQUEST_METHOD': 'POST',
         'CONTENT_TYPE': 'text/plain',
         'HTTP_ACCEPT': 'image/png'}), handlers)
    assert rv == (post_handlers[1], vary_accept)

    assert_raises(NotAcceptable, resolve_handler, Request(
        {'REQUEST_METHOD': 'POST',
         'CONTENT_TYPE': 'text/plain',
         'HTTP_ACCEPT': 'application/json'}), handlers)

    rv = resolve_handler(Request(
        {'REQUEST_METHOD': 'PUT',
         'CONTENT_TYPE': 'application/json'}), handlers)
    assert rv == (put_handlers[1], vary_content_type)

    req = Request({'REQUEST_METHOD': 'POST', 'CONTENT_TYPE': 'application/xml'})
    req._add_context(root=None, mapper=None, route=Route('/', None, name=':test'))
    rv = resolve_handler(req, handlers)
    assert rv == (post_handlers_test_view[1], vary_both)


def test_make_response():
    assert_raises(TypeError, make_response, None)

    res = make_response('')
    assert res.code == 200
    assert res.body == ''

    orig = Response(200, body='test')
    res = make_response(orig)
    assert res is orig


def test_resource_from_url():
    resource1 = Resource()

    @resource1.from_url
    def from_url_1(request, ctx, a, b):
        return {'x': 1, 'y': 2}

    @resource1.get
    def handler1(request, x, y):
        resource1.args = (x, y)
        return ok()

    resource2 = Resource()
    resource2.get(lambda req, x, y: ok())

    @resource2.from_url
    def from_url_2(request, a, b):
        return {'x': 3, 'y': 4}

    @resource2.get
    def handler2(request, x, y):
        resource2.args = (x, y)
        return ok()

    req = Request({'REQUEST_METHOD': 'GET'})
    req.routing_args.update({'a': 1, 'b': 2})
    ctx = Context()
    resource1(req, ctx)
    assert resource1.args == (1, 2)

    req = Request({'REQUEST_METHOD': 'GET'})
    req.routing_args.update({'a': 1, 'b': 2})
    ctx = Context()
    resource2(req, ctx)
    assert resource2.args == (3, 4)



from __future__ import absolute_import

from .http import cache_control
from .mapper import Mapper
from .request import Request
from .resource import Resource, get, post, put, delete, patch, options
from .response import Response, Entity, \
        response, ok, created, no_content, redirect
from .static import StaticFile, StaticDirectory

__version__ = '0.0.1'



from rhino.vendor import mimeparse

def test_mimeparse_q0():
    """Test for a bug in the official mimeparse release that is fixed in our
    vendored version."""
    assert mimeparse.parse_media_range('image/*; q=0.0') == ('image', '*', {'q': '0.0'})



from rhino import Mapper, Resource, get, redirect
import storage

wiki = storage.Storage('contents')

edit_page = Resource()
history_page = Resource()

@get
def display(request, name):
    return wiki.render_page(name)

@edit_page.get
def edit_form(request, name):
    return wiki.render_edit_form(name)

@edit_page.post
def edit(request, name):
    if request.form.get('submit'):
        wiki.store_page(name, request.form['content'])
    return redirect('/%s' % name)

@history_page.get
def history(request, name):
    return wiki.render_history_form(name)

@history_page.post
def revert(request, name):
    version = request.form.get('version')
    if request.form.get('submit') and version:
        wiki.revert_page(name, version)
    return redirect('/%s' % name)

app = Mapper(ranges={'wikiname': storage.wikiname_re.pattern})
app.default_content_type='text/html; charset=utf-8'

app.add('/', lambda req: redirect('/FrontPage'))
app.add('/{name:wikiname}', display)
app.add('/{name:wikiname}/edit', edit_page)
app.add('/{name:wikiname}/history', history_page)

if __name__ == '__main__':
    app.start_server(host='127.0.0.1', port=8080)



from rhino.mapper import Mapper
from rhino.urls import request_context, build_url


def test_build_url():
    mapper = Mapper()
    mapper.add('/foo', 1, 'foo')

    context = [request_context('', mapper, mapper.routes[0])]

    assert build_url(context, '/')    == '/'
    assert build_url(context, '.')    == '/foo'
    assert build_url(context, 'foo')  == '/foo'
    assert build_url(context, '.foo') == '/foo'
    assert build_url(context, '/foo') == '/foo'
    assert build_url(context, 1)      == '/foo'


def test_build_url_nested():
    mapper1 = Mapper()
    mapper2 = Mapper()
    mapper2.add('/bar', 1, 'bar')
    mapper1.add('/foo|', mapper2, 'foo')

    context = [request_context('',     mapper1, mapper1.routes[0]),
               request_context('/foo', mapper2, mapper2.routes[0])]

    assert build_url(context, '/')         == '/'
    assert build_url(context, '/foo')      == '/foo'
    assert build_url(context, '/foo.bar')  == '/foo/bar'
    assert build_url(context, '.')         == '/foo/bar'
    assert build_url(context, 'bar')       == '/foo/bar'
    assert build_url(context, '.bar')      == '/foo/bar'
    assert build_url(context, '..foo')     == '/foo'
    assert build_url(context, '..foo.bar') == '/foo/bar'
